@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2016 The University of Tennessee and The University
+ * Copyright (c) 2004-2021 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -17,6 +17,8 @@
  * Copyright (c) 2014-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2017      IBM Corporation. All rights reserved.
+ * Copyright (c) 2021      Amazon.com, Inc. or its affiliates.  All Rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -29,6 +31,7 @@
 #include "mpi.h"
 #include "ompi/constants.h"
 #include "ompi/datatype/ompi_datatype.h"
+#include "opal/datatype/opal_convertor_internal.h"
 #include "ompi/communicator/communicator.h"
 #include "ompi/mca/coll/coll.h"
 #include "ompi/mca/coll/base/coll_tags.h"
@@ -37,85 +40,130 @@
 #include "coll_base_topo.h"
 #include "coll_base_util.h"
 
+/*
+ * We want to minimize the amount of temporary memory needed while allowing as many ranks
+ * to exchange data simultaneously. We use a variation of the ring algorithm, where in a
+ * single step a process exchange the data with both neighbors at distance k (on the left
+ * and the right on a logical ring topology). With this approach we need to pack the data
+ * for a single of the two neighbors, as we can then use the original buffer (and datatype
+ * and count) to send the data to the other.
+ */
 int
 mca_coll_base_alltoallv_intra_basic_inplace(const void *rbuf, const int *rcounts, const int *rdisps,
                                             struct ompi_datatype_t *rdtype,
                                             struct ompi_communicator_t *comm,
                                             mca_coll_base_module_t *module)
 {
-    int i, j, size, rank, err=MPI_SUCCESS;
-    char *allocated_buffer, *tmp_buffer;
-    size_t max_size;
-    ptrdiff_t ext, gap = 0;
+    int i, size, rank, left, right, err = MPI_SUCCESS, line;
+    ptrdiff_t extent;
+    ompi_request_t *req = MPI_REQUEST_NULL;
+    char *tmp_buffer;
+    size_t packed_size = 0, max_size, type_size;
+    opal_convertor_t convertor;
 
     /* Initialize. */
 
     size = ompi_comm_size(comm);
     rank = ompi_comm_rank(comm);
+    ompi_datatype_type_size(rdtype, &type_size);
 
-    /* If only one process, we're done. */
-    if (1 == size) {
-        return MPI_SUCCESS;
-    }
-    /* Find the largest receive amount */
-    ompi_datatype_type_extent (rdtype, &ext);
     for (i = 0, max_size = 0 ; i < size ; ++i) {
         if (i == rank) {
             continue;
         }
-        size_t size = opal_datatype_span(&rdtype->super, rcounts[i], &gap);
-        max_size = size > max_size ? size : max_size;
+        packed_size = rcounts[i] * type_size;
+        max_size = packed_size > max_size ? packed_size : max_size;
     }
-    /* The gap will always be the same as we are working on the same datatype */
 
-    if (OPAL_UNLIKELY(0 == max_size)) {
+    /* Easy way out */
+    if ((1 == size) || (0 == max_size) ) {
         return MPI_SUCCESS;
     }
 
-    /* Allocate a temporary buffer */
-    allocated_buffer = calloc (max_size, 1);
-    if (NULL == allocated_buffer) {
-        return OMPI_ERR_OUT_OF_RESOURCE;
-    }
-    tmp_buffer = allocated_buffer - gap;
+    /* Find the largest amount of packed send/recv data among all peers where
+     * we need to pack before the send.
+     */
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
+    for (i = 1 ; i <= (size >> 1) ; ++i) {
+        right = (rank + i) % size;
+        ompi_proc_t *ompi_proc = ompi_comm_peer_lookup(comm, right);
 
-    /* Initiate all send/recv to/from others. */
-    /* in-place alltoallv slow algorithm (but works) */
-    for (i = 0 ; i < size ; ++i) {
-        for (j = i+1 ; j < size ; ++j) {
-            if (i == rank && 0 != rcounts[j]) {
-                /* Copy the data into the temporary buffer */
-                err = ompi_datatype_copy_content_same_ddt (rdtype, rcounts[j],
-                                                           tmp_buffer, (char *) rbuf + rdisps[j] * ext);
-                if (MPI_SUCCESS != err) { goto error_hndl; }
-
-                /* Exchange data with the peer */
-                err = ompi_coll_base_sendrecv_actual((void *) tmp_buffer,  rcounts[j], rdtype,
-                                                     j, MCA_COLL_BASE_TAG_ALLTOALLV,
-                                                     (char *)rbuf + rdisps[j] * ext, rcounts[j], rdtype,
-                                                     j, MCA_COLL_BASE_TAG_ALLTOALLV,
-                                                     comm, MPI_STATUS_IGNORE);
-                if (MPI_SUCCESS != err) { goto error_hndl; }
-            } else if (j == rank && 0 != rcounts[i]) {
-                /* Copy the data into the temporary buffer */
-                err = ompi_datatype_copy_content_same_ddt (rdtype, rcounts[i],
-                                                           tmp_buffer, (char *) rbuf + rdisps[i] * ext);
-                if (MPI_SUCCESS != err) { goto error_hndl; }
-
-                /* Exchange data with the peer */
-                err = ompi_coll_base_sendrecv_actual((void *) tmp_buffer,  rcounts[i], rdtype,
-                                                     i, MCA_COLL_BASE_TAG_ALLTOALLV,
-                                                     (char *) rbuf + rdisps[i] * ext, rcounts[i], rdtype,
-                                                     i, MCA_COLL_BASE_TAG_ALLTOALLV,
-                                                     comm, MPI_STATUS_IGNORE);
-                if (MPI_SUCCESS != err) { goto error_hndl; }
-            }
+        if( OPAL_UNLIKELY(opal_local_arch != ompi_proc->super.proc_convertor->master->remote_arch))  {
+            packed_size = opal_datatype_compute_remote_size(&rdtype->super,
+                                                            ompi_proc->super.proc_convertor->master->remote_sizes);
+            packed_size *= rcounts[right];
+            max_size = packed_size > max_size ? packed_size : max_size;
         }
     }
+#endif  /* OPAL_ENABLE_HETEROGENEOUS_SUPPORT */
+
+    ompi_datatype_type_extent(rdtype, &extent);
+
+    /* Allocate a temporary buffer */
+    tmp_buffer = calloc (max_size, 1);
+    if( NULL == tmp_buffer) { err = OMPI_ERR_OUT_OF_RESOURCE; line = __LINE__; goto error_hndl; }
+
+    for (i = 1 ; i <= (size >> 1) ; ++i) {
+        struct iovec iov = {.iov_base = tmp_buffer, .iov_len = max_size};
+        uint32_t iov_count = 1;
+
+        right = (rank + i) % size;
+        left  = (rank + size - i) % size;
+
+        if( 0 != rcounts[right] ) {  /* nothing to exchange with the peer on the right */
+            ompi_proc_t *right_proc = ompi_comm_peer_lookup(comm, right);
+            opal_convertor_clone(right_proc->super.proc_convertor, &convertor, 0);
+            opal_convertor_prepare_for_send(&convertor, &rdtype->super, rcounts[right],
+                                            (char *) rbuf + rdisps[right] * extent);
+            packed_size = max_size;
+            err = opal_convertor_pack(&convertor, &iov, &iov_count, &packed_size);
+            if (1 != err) { goto error_hndl; }
+
+            /* Receive data from the right */
+            err = MCA_PML_CALL(irecv ((char *) rbuf + rdisps[right] * extent, rcounts[right], rdtype,
+                                      right, MCA_COLL_BASE_TAG_ALLTOALLV, comm, &req));
+            if (MPI_SUCCESS != err) { goto error_hndl; }
+        }
+
+        if( (left != right) && (0 != rcounts[left]) ) {
+            /* Send data to the left */
+            err = MCA_PML_CALL(send ((char *) rbuf + rdisps[left] * extent, rcounts[left], rdtype,
+                                     left, MCA_COLL_BASE_TAG_ALLTOALLV, MCA_PML_BASE_SEND_STANDARD,
+                                     comm));
+            if (MPI_SUCCESS != err) { goto error_hndl; }
+
+            err = ompi_request_wait (&req, MPI_STATUSES_IGNORE);
+            if (MPI_SUCCESS != err) { goto error_hndl; }
+
+            /* Receive data from the left */
+            err = MCA_PML_CALL(irecv ((char *) rbuf + rdisps[left] * extent, rcounts[left], rdtype,
+                                      left, MCA_COLL_BASE_TAG_ALLTOALLV, comm, &req));
+            if (MPI_SUCCESS != err) { goto error_hndl; }
+        }
+
+        if( 0 != rcounts[right] ) {  /* nothing to exchange with the peer on the right */
+            /* Send data to the right */
+            err = MCA_PML_CALL(send ((char *) tmp_buffer,  packed_size, MPI_PACKED,
+                                     right, MCA_COLL_BASE_TAG_ALLTOALLV, MCA_PML_BASE_SEND_STANDARD,
+                                     comm));
+            if (MPI_SUCCESS != err) { goto error_hndl; }
+        }
+
+        err = ompi_request_wait (&req, MPI_STATUSES_IGNORE);
+        if (MPI_SUCCESS != err) { goto error_hndl; }
+    }
 
  error_hndl:
     /* Free the temporary buffer */
-    free (allocated_buffer);
+    if( NULL != tmp_buffer )
+        free (tmp_buffer);
+
+    if( MPI_SUCCESS != err ) {
+        OPAL_OUTPUT((ompi_coll_base_framework.framework_output,
+                     "%s:%4d\tError occurred %d, rank %2d", __FILE__, line, err,
+                     rank));
+        (void)line;  // silence compiler warning
+    }
 
     /* All done */
     return err;
@@ -276,6 +324,17 @@ ompi_coll_base_alltoallv_intra_basic_linear(const void *sbuf, const int *scounts
     err = ompi_request_wait_all(nreqs, reqs, MPI_STATUSES_IGNORE);
 
  err_hndl:
+    /* find a real error code */
+    if (MPI_ERR_IN_STATUS == err) {
+        for( i = 0; i < nreqs; i++ ) {
+            if (MPI_REQUEST_NULL == reqs[i]) continue;
+            if (MPI_ERR_PENDING == reqs[i]->req_status.MPI_ERROR) continue;
+            if (reqs[i]->req_status.MPI_ERROR != MPI_SUCCESS) {
+                err = reqs[i]->req_status.MPI_ERROR;
+                break;
+            }
+        }
+    }
     /* Free the requests in all cases as they are persistent */
     ompi_coll_base_free_reqs(reqs, nreqs);
 
