@@ -13,7 +13,7 @@
  * Copyright (c) 2008-2015 University of Houston. All rights reserved.
  * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
  *                         reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2016-2017 IBM Corporation. All rights reserved.
  * $COPYRIGHT$
@@ -34,7 +34,7 @@
 
 BEGIN_C_DECLS
 
-struct mca_io_ompio_file_t;
+struct ompio_file_t;
 
 /*
  * Macro for use in components that are of type coll
@@ -74,11 +74,11 @@ typedef int (*mca_fs_base_component_init_query_1_0_0_fn_t)
      bool enable_mpi_threads);
 
 typedef struct mca_fs_base_module_1_0_0_t *
-(*mca_fs_base_component_file_query_1_0_0_fn_t) (struct mca_io_ompio_file_t *file,
+(*mca_fs_base_component_file_query_1_0_0_fn_t) (struct ompio_file_t *file,
                                                 int *priority);
 
 typedef int (*mca_fs_base_component_file_unquery_1_0_0_fn_t)
-    (struct mca_io_ompio_file_t *file);
+    (struct ompio_file_t *file);
 
 /*
  * ****************** component struct ******************************
@@ -105,23 +105,23 @@ typedef struct mca_fs_base_component_2_0_0_t mca_fs_base_component_t;
  */
 
 typedef int (*mca_fs_base_module_init_1_0_0_fn_t)
-(struct mca_io_ompio_file_t *file);
+(struct ompio_file_t *file);
 
 typedef int (*mca_fs_base_module_finalize_1_0_0_fn_t)
-(struct mca_io_ompio_file_t *file);
+(struct ompio_file_t *file);
 
 typedef int (*mca_fs_base_module_file_open_fn_t)(
     struct ompi_communicator_t *comm, const char *filename, int amode,
-    struct opal_info_t *info, struct mca_io_ompio_file_t *fh);
-typedef int (*mca_fs_base_module_file_close_fn_t)(struct mca_io_ompio_file_t *fh);
+    struct opal_info_t *info, struct ompio_file_t *fh);
+typedef int (*mca_fs_base_module_file_close_fn_t)(struct ompio_file_t *fh);
 typedef int (*mca_fs_base_module_file_delete_fn_t)(
     char *filename, struct opal_info_t *info);
 typedef int (*mca_fs_base_module_file_set_size_fn_t)
-    (struct mca_io_ompio_file_t *fh, OMPI_MPI_OFFSET_TYPE size);
+    (struct ompio_file_t *fh, OMPI_MPI_OFFSET_TYPE size);
 typedef int (*mca_fs_base_module_file_get_size_fn_t)
-    (struct mca_io_ompio_file_t *fh, OMPI_MPI_OFFSET_TYPE *size);
+    (struct ompio_file_t *fh, OMPI_MPI_OFFSET_TYPE *size);
 typedef int (*mca_fs_base_module_file_sync_fn_t)
-    (struct mca_io_ompio_file_t *fh);
+    (struct ompio_file_t *fh);
 
 /*
  * ***********************************************************************
