@@ -444,8 +444,8 @@ void ADIOI_GPFS_ReadStridedColl(ADIO_File fd, void *buf, int count,
     ADIOI_Free(count_my_req_per_proc);
     for (i=0; i<nprocs; i++) {
 	if (my_req[i].count) {
-	    ADIOI_Free(my_req[i].offsets);
-	    ADIOI_Free(my_req[i].lens);
+	    if (my_req[i].offsets) { ADIOI_Free(my_req[i].offsets); }
+	    if (my_req[i].lens) { ADIOI_Free(my_req[i].lens); }
 	}
     }
     ADIOI_Free(my_req);
@@ -469,9 +469,9 @@ void ADIOI_GPFS_ReadStridedColl(ADIO_File fd, void *buf, int count,
     /* free all memory allocated for collective I/O */
     for (i=0; i<nprocs; i++) {
 	if (others_req[i].count) {
-	    ADIOI_Free(others_req[i].offsets);
-	    ADIOI_Free(others_req[i].lens);
-	    ADIOI_Free(others_req[i].mem_ptrs);
+	    if (others_req[i].offsets) { ADIOI_Free(others_req[i].offsets); }
+	    if (others_req[i].lens) { ADIOI_Free(others_req[i].lens); }
+	    if (others_req[i].mem_ptrs) { ADIOI_Free(others_req[i].mem_ptrs); }
 	}
     }
     ADIOI_Free(others_req);
