@@ -37,7 +37,7 @@
  * This macro is for (out op in).
  */
 #define OP_FUNC(name, type_name, type, op) \
-  static void ompi_op_base_2buff_##name##_##type_name(void *in, void *out, int *count, \
+  static void ompi_op_base_2buff_##name##_##type_name(const void *in, void *out, int *count, \
                                                       struct ompi_datatype_t **dtype, \
                                                       struct ompi_op_base_module_1_0_0_t *module) \
   {                                                                      \
@@ -57,7 +57,7 @@
  * This macro is for (out = op(out, in))
  */
 #define FUNC_FUNC(name, type_name, type) \
-  static void ompi_op_base_2buff_##name##_##type_name(void *in, void *out, int *count, \
+  static void ompi_op_base_2buff_##name##_##type_name(const void *in, void *out, int *count, \
                                                 struct ompi_datatype_t **dtype, \
                                                 struct ompi_op_base_module_1_0_0_t *module) \
   {                                                                      \
@@ -85,7 +85,7 @@
   } ompi_op_predefined_##type_name##_t;
 
 #define LOC_FUNC(name, type_name, op) \
-    static void ompi_op_base_2buff_##name##_##type_name(void *in, void *out, int *count, \
+    static void ompi_op_base_2buff_##name##_##type_name(const void *in, void *out, int *count, \
                                                         struct ompi_datatype_t **dtype, \
                                                         struct ompi_op_base_module_1_0_0_t *module) \
     {                                                                   \
@@ -102,6 +102,49 @@
         }                                                               \
     }
 
+/*
+ * Define a function to calculate sum of complex numbers using a real
+ * number floating-point type (float, double, etc.).  This macro is used
+ * when the compiler supports a real number floating-point type but does
+ * not supports the corresponding complex number type.
+ */
+#define COMPLEX_SUM_FUNC(type_name, type) \
+  static void ompi_op_base_2buff_sum_##type_name(const void *in, void *out, int *count, \
+                                                 struct ompi_datatype_t **dtype, \
+                                                 struct ompi_op_base_module_1_0_0_t *module) \
+  {                                                                      \
+      int i;                                                             \
+      type (*a)[2] = (type (*)[2]) in;                                   \
+      type (*b)[2] = (type (*)[2]) out;                                  \
+      for (i = 0; i < *count; ++i, ++a, ++b) {                           \
+          (*b)[0] += (*a)[0];                                            \
+          (*b)[1] += (*a)[1];                                            \
+      }                                                                  \
+  }
+
+/*
+ * Define a function to calculate product of complex numbers using a real
+ * number floating-point type (float, double, etc.).  This macro is used
+ * when the compiler supports a real number floating-point type but does
+ * not supports the corresponding complex number type.
+ */
+#define COMPLEX_PROD_FUNC(type_name, type) \
+  static void ompi_op_base_2buff_prod_##type_name(const void *in, void *out, int *count, \
+                                                  struct ompi_datatype_t **dtype, \
+                                                  struct ompi_op_base_module_1_0_0_t *module) \
+  {                                                                      \
+      int i;                                                             \
+      type (*a)[2] = (type (*)[2]) in;                                   \
+      type (*b)[2] = (type (*)[2]) out;                                  \
+      type c[2];                                                         \
+      for (i = 0; i < *count; ++i, ++a, ++b) {                           \
+          c[0] = (*a)[0] * (*b)[0] - (*a)[1] * (*b)[1];                  \
+          c[1] = (*a)[0] * (*b)[1] + (*a)[1] * (*b)[0];                  \
+          (*b)[0] = c[0];                                                \
+          (*b)[1] = c[1];                                                \
+      }                                                                  \
+  }
+
 /*************************************************************************
  * Max
  *************************************************************************/
@@ -117,6 +160,9 @@ FUNC_FUNC(max,  int32_t,  int32_t)
 FUNC_FUNC(max, uint32_t, uint32_t)
 FUNC_FUNC(max,  int64_t,  int64_t)
 FUNC_FUNC(max, uint64_t, uint64_t)
+FUNC_FUNC(max,  long,  long)
+FUNC_FUNC(max,  unsigned_long,  long)
+
 /* Fortran integer */
 #if OMPI_HAVE_FORTRAN_INTEGER
 FUNC_FUNC(max, fortran_integer, ompi_fortran_integer_t)
@@ -177,6 +223,9 @@ FUNC_FUNC(min,  int32_t,  int32_t)
 FUNC_FUNC(min, uint32_t, uint32_t)
 FUNC_FUNC(min,  int64_t,  int64_t)
 FUNC_FUNC(min, uint64_t, uint64_t)
+FUNC_FUNC(min,  long,  long)
+FUNC_FUNC(min,  unsigned_long,  long)
+
 /* Fortran integer */
 #if OMPI_HAVE_FORTRAN_INTEGER
 FUNC_FUNC(min, fortran_integer, ompi_fortran_integer_t)
@@ -234,6 +283,9 @@ OP_FUNC(sum,  int32_t,  int32_t, +=)
 OP_FUNC(sum, uint32_t, uint32_t, +=)
 OP_FUNC(sum,  int64_t,  int64_t, +=)
 OP_FUNC(sum, uint64_t, uint64_t, +=)
+OP_FUNC(sum,  long,  long, +=)
+OP_FUNC(sum,  unsigned_long,  long, +=)
+
 /* Fortran integer */
 #if OMPI_HAVE_FORTRAN_INTEGER
 OP_FUNC(sum, fortran_integer, ompi_fortran_integer_t, +=)
@@ -301,6 +353,9 @@ OP_FUNC(prod,  int32_t,  int32_t, *=)
 OP_FUNC(prod, uint32_t, uint32_t, *=)
 OP_FUNC(prod,  int64_t,  int64_t, *=)
 OP_FUNC(prod, uint64_t, uint64_t, *=)
+OP_FUNC(prod,  long,  long, *=)
+OP_FUNC(prod,  unsigned_long,  long, *=)
+
 /* Fortran integer */
 #if OMPI_HAVE_FORTRAN_INTEGER
 OP_FUNC(prod, fortran_integer, ompi_fortran_integer_t, *=)
@@ -370,6 +425,9 @@ FUNC_FUNC(land,  int32_t,  int32_t)
 FUNC_FUNC(land, uint32_t, uint32_t)
 FUNC_FUNC(land,  int64_t,  int64_t)
 FUNC_FUNC(land, uint64_t, uint64_t)
+FUNC_FUNC(land,  long,  long)
+FUNC_FUNC(land,  unsigned_long,  long)
+
 /* Logical */
 #if OMPI_HAVE_FORTRAN_LOGICAL
 FUNC_FUNC(land, fortran_logical, ompi_fortran_logical_t)
@@ -392,6 +450,9 @@ FUNC_FUNC(lor,  int32_t,  int32_t)
 FUNC_FUNC(lor, uint32_t, uint32_t)
 FUNC_FUNC(lor,  int64_t,  int64_t)
 FUNC_FUNC(lor, uint64_t, uint64_t)
+FUNC_FUNC(lor,  long,  long)
+FUNC_FUNC(lor,  unsigned_long,  long)
+
 /* Logical */
 #if OMPI_HAVE_FORTRAN_LOGICAL
 FUNC_FUNC(lor, fortran_logical, ompi_fortran_logical_t)
@@ -414,6 +475,10 @@ FUNC_FUNC(lxor,  int32_t,  int32_t)
 FUNC_FUNC(lxor, uint32_t, uint32_t)
 FUNC_FUNC(lxor,  int64_t,  int64_t)
 FUNC_FUNC(lxor, uint64_t, uint64_t)
+FUNC_FUNC(lxor,  long,  long)
+FUNC_FUNC(lxor,  unsigned_long,  long)
+
+
 /* Logical */
 #if OMPI_HAVE_FORTRAN_LOGICAL
 FUNC_FUNC(lxor, fortran_logical, ompi_fortran_logical_t)
@@ -436,6 +501,9 @@ FUNC_FUNC(band,  int32_t,  int32_t)
 FUNC_FUNC(band, uint32_t, uint32_t)
 FUNC_FUNC(band,  int64_t,  int64_t)
 FUNC_FUNC(band, uint64_t, uint64_t)
+FUNC_FUNC(band,  long,  long)
+FUNC_FUNC(band,  unsigned_long,  long)
+
 /* Fortran integer */
 #if OMPI_HAVE_FORTRAN_INTEGER
 FUNC_FUNC(band, fortran_integer, ompi_fortran_integer_t)
@@ -473,6 +541,9 @@ FUNC_FUNC(bor,  int32_t,  int32_t)
 FUNC_FUNC(bor, uint32_t, uint32_t)
 FUNC_FUNC(bor,  int64_t,  int64_t)
 FUNC_FUNC(bor, uint64_t, uint64_t)
+FUNC_FUNC(bor,  long,  long)
+FUNC_FUNC(bor,  unsigned_long,  long)
+
 /* Fortran integer */
 #if OMPI_HAVE_FORTRAN_INTEGER
 FUNC_FUNC(bor, fortran_integer, ompi_fortran_integer_t)
@@ -510,6 +581,9 @@ FUNC_FUNC(bxor,  int32_t,  int32_t)
 FUNC_FUNC(bxor, uint32_t, uint32_t)
 FUNC_FUNC(bxor,  int64_t,  int64_t)
 FUNC_FUNC(bxor, uint64_t, uint64_t)
+FUNC_FUNC(bxor,  long,  long)
+FUNC_FUNC(bxor,  unsigned_long,  long)
+
 /* Fortran integer */
 #if OMPI_HAVE_FORTRAN_INTEGER
 FUNC_FUNC(bxor, fortran_integer, ompi_fortran_integer_t)
@@ -553,6 +627,7 @@ LOC_STRUCT(short_int, short, int)
 #if HAVE_LONG_DOUBLE
 LOC_STRUCT(long_double_int, long double, int)
 #endif
+LOC_STRUCT(unsigned_long, unsigned long, int)
 
 /*************************************************************************
  * Max location
@@ -604,8 +679,8 @@ LOC_FUNC(minloc, long_double_int, <)
  *    routines, needed for some optimizations.
  */
 #define OP_FUNC_3BUF(name, type_name, type, op) \
-    static void ompi_op_base_3buff_##name##_##type_name(void * restrict in1,   \
-                                                        void * restrict in2, void * restrict out, int *count, \
+    static void ompi_op_base_3buff_##name##_##type_name(const void * restrict in1,   \
+                                                        const void * restrict in2, void * restrict out, int *count, \
                                                         struct ompi_datatype_t **dtype, \
                                                         struct ompi_op_base_module_1_0_0_t *module) \
     {                                                                   \
@@ -626,8 +701,8 @@ LOC_FUNC(minloc, long_double_int, <)
  * This macro is for (out = op(in1, in2))
  */
 #define FUNC_FUNC_3BUF(name, type_name, type)                           \
-    static void ompi_op_base_3buff_##name##_##type_name(void * restrict in1, \
-                                                        void * restrict in2, void * restrict out, int *count, \
+    static void ompi_op_base_3buff_##name##_##type_name(const void * restrict in1, \
+                                                        const void * restrict in2, void * restrict out, int *count, \
                                                         struct ompi_datatype_t **dtype, \
                                                         struct ompi_op_base_module_1_0_0_t *module) \
     {                                                                   \
@@ -659,8 +734,8 @@ LOC_FUNC(minloc, long_double_int, <)
 */
 
 #define LOC_FUNC_3BUF(name, type_name, op) \
-  static void ompi_op_base_3buff_##name##_##type_name(void * restrict in1,      \
-                                                      void * restrict in2, void * restrict out, int *count, \
+  static void ompi_op_base_3buff_##name##_##type_name(const void * restrict in1,      \
+                                                      const void * restrict in2, void * restrict out, int *count, \
                                                       struct ompi_datatype_t **dtype, \
                                                       struct ompi_op_base_module_1_0_0_t *module) \
   {                                                                     \
@@ -682,6 +757,50 @@ LOC_FUNC(minloc, long_double_int, <)
       }                                                                 \
   }
 
+/*
+ * Define a function to calculate sum of complex numbers using a real
+ * number floating-point type (float, double, etc.).  This macro is used
+ * when the compiler supports a real number floating-point type but does
+ * not supports the corresponding complex number type.
+ */
+#define COMPLEX_SUM_FUNC_3BUF(type_name, type) \
+  static void ompi_op_base_3buff_sum_##type_name(const void * restrict in1,    \
+                                                 const void * restrict in2, void * restrict out, int *count, \
+                                                 struct ompi_datatype_t **dtype, \
+                                                 struct ompi_op_base_module_1_0_0_t *module) \
+  {                                                                      \
+      int i;                                                             \
+      type (*a1)[2] = (type (*)[2]) in1;                                 \
+      type (*a2)[2] = (type (*)[2]) in2;                                 \
+      type (*b)[2] = (type (*)[2]) out;                                  \
+      for (i = 0; i < *count; ++i, ++a1, ++a2, ++b) {                    \
+          (*b)[0] = (*a1)[0] + (*a2)[0];                                 \
+          (*b)[1] = (*a1)[1] + (*a2)[1];                                 \
+      }                                                                  \
+  }
+
+/*
+ * Define a function to calculate product of complex numbers using a real
+ * number floating-point type (float, double, etc.).  This macro is used
+ * when the compiler supports a real number floating-point type but does
+ * not supports the corresponding complex number type.
+ */
+#define COMPLEX_PROD_FUNC_3BUF(type_name, type) \
+  static void ompi_op_base_3buff_prod_##type_name(const void * restrict in1,   \
+                                                  const void * restrict in2, void * restrict out, int *count, \
+                                                  struct ompi_datatype_t **dtype, \
+                                                  struct ompi_op_base_module_1_0_0_t *module) \
+  {                                                                      \
+      int i;                                                             \
+      type (*a1)[2] = (type (*)[2]) in1;                                 \
+      type (*a2)[2] = (type (*)[2]) in2;                                 \
+      type (*b)[2] = (type (*)[2]) out;                                  \
+      for (i = 0; i < *count; ++i, ++a1, ++a2, ++b) {                    \
+          (*b)[0] = (*a1)[0] * (*a2)[0] - (*a1)[1] * (*a2)[1];           \
+          (*b)[1] = (*a1)[0] * (*a2)[1] + (*a1)[1] * (*a2)[0];           \
+      }                                                                  \
+  }
+
 /*************************************************************************
  * Max
  *************************************************************************/
@@ -697,6 +816,9 @@ FUNC_FUNC_3BUF(max,  int32_t,  int32_t)
 FUNC_FUNC_3BUF(max, uint32_t, uint32_t)
 FUNC_FUNC_3BUF(max,  int64_t,  int64_t)
 FUNC_FUNC_3BUF(max, uint64_t, uint64_t)
+FUNC_FUNC_3BUF(max,  long,  long)
+FUNC_FUNC_3BUF(max,  unsigned_long,  long)
+
 /* Fortran integer */
 #if OMPI_HAVE_FORTRAN_INTEGER
 FUNC_FUNC_3BUF(max, fortran_integer, ompi_fortran_integer_t)
@@ -757,6 +879,9 @@ FUNC_FUNC_3BUF(min,  int32_t,  int32_t)
 FUNC_FUNC_3BUF(min, uint32_t, uint32_t)
 FUNC_FUNC_3BUF(min,  int64_t,  int64_t)
 FUNC_FUNC_3BUF(min, uint64_t, uint64_t)
+FUNC_FUNC_3BUF(min,  long,  long)
+FUNC_FUNC_3BUF(min,  unsigned_long,  long)
+
 /* Fortran integer */
 #if OMPI_HAVE_FORTRAN_INTEGER
 FUNC_FUNC_3BUF(min, fortran_integer, ompi_fortran_integer_t)
@@ -814,6 +939,9 @@ OP_FUNC_3BUF(sum,  int32_t,  int32_t, +)
 OP_FUNC_3BUF(sum, uint32_t, uint32_t, +)
 OP_FUNC_3BUF(sum,  int64_t,  int64_t, +)
 OP_FUNC_3BUF(sum, uint64_t, uint64_t, +)
+OP_FUNC_3BUF(sum,  long,  long, +)
+OP_FUNC_3BUF(sum,  unsigned_long,  long, +)
+
 /* Fortran integer */
 #if OMPI_HAVE_FORTRAN_INTEGER
 OP_FUNC_3BUF(sum, fortran_integer, ompi_fortran_integer_t, +)
@@ -881,6 +1009,9 @@ OP_FUNC_3BUF(prod,  int32_t,  int32_t, *)
 OP_FUNC_3BUF(prod, uint32_t, uint32_t, *)
 OP_FUNC_3BUF(prod,  int64_t,  int64_t, *)
 OP_FUNC_3BUF(prod, uint64_t, uint64_t, *)
+OP_FUNC_3BUF(prod,  long,  long, *)
+OP_FUNC_3BUF(prod,  unsigned_long,  long, *)
+
 /* Fortran integer */
 #if OMPI_HAVE_FORTRAN_INTEGER
 OP_FUNC_3BUF(prod, fortran_integer, ompi_fortran_integer_t, *)
@@ -950,6 +1081,9 @@ FUNC_FUNC_3BUF(land,  int32_t,  int32_t)
 FUNC_FUNC_3BUF(land, uint32_t, uint32_t)
 FUNC_FUNC_3BUF(land,  int64_t,  int64_t)
 FUNC_FUNC_3BUF(land, uint64_t, uint64_t)
+FUNC_FUNC_3BUF(land,  long,  long)
+FUNC_FUNC_3BUF(land,  unsigned_long,  long)
+
 /* Logical */
 #if OMPI_HAVE_FORTRAN_LOGICAL
 FUNC_FUNC_3BUF(land, fortran_logical, ompi_fortran_logical_t)
@@ -972,6 +1106,9 @@ FUNC_FUNC_3BUF(lor,  int32_t,  int32_t)
 FUNC_FUNC_3BUF(lor, uint32_t, uint32_t)
 FUNC_FUNC_3BUF(lor,  int64_t,  int64_t)
 FUNC_FUNC_3BUF(lor, uint64_t, uint64_t)
+FUNC_FUNC_3BUF(lor,  long,  long)
+FUNC_FUNC_3BUF(lor,  unsigned_long,  long)
+
 /* Logical */
 #if OMPI_HAVE_FORTRAN_LOGICAL
 FUNC_FUNC_3BUF(lor, fortran_logical, ompi_fortran_logical_t)
@@ -994,6 +1131,9 @@ FUNC_FUNC_3BUF(lxor,  int32_t,  int32_t)
 FUNC_FUNC_3BUF(lxor, uint32_t, uint32_t)
 FUNC_FUNC_3BUF(lxor,  int64_t,  int64_t)
 FUNC_FUNC_3BUF(lxor, uint64_t, uint64_t)
+FUNC_FUNC_3BUF(lxor,  long,  long)
+FUNC_FUNC_3BUF(lxor,  unsigned_long,  long)
+
 /* Logical */
 #if OMPI_HAVE_FORTRAN_LOGICAL
 FUNC_FUNC_3BUF(lxor, fortran_logical, ompi_fortran_logical_t)
@@ -1016,6 +1156,9 @@ FUNC_FUNC_3BUF(band,  int32_t,  int32_t)
 FUNC_FUNC_3BUF(band, uint32_t, uint32_t)
 FUNC_FUNC_3BUF(band,  int64_t,  int64_t)
 FUNC_FUNC_3BUF(band, uint64_t, uint64_t)
+FUNC_FUNC_3BUF(band,  long,  long)
+FUNC_FUNC_3BUF(band,  unsigned_long,  long)
+
 /* Fortran integer */
 #if OMPI_HAVE_FORTRAN_INTEGER
 FUNC_FUNC_3BUF(band, fortran_integer, ompi_fortran_integer_t)
@@ -1053,6 +1196,9 @@ FUNC_FUNC_3BUF(bor,  int32_t,  int32_t)
 FUNC_FUNC_3BUF(bor, uint32_t, uint32_t)
 FUNC_FUNC_3BUF(bor,  int64_t,  int64_t)
 FUNC_FUNC_3BUF(bor, uint64_t, uint64_t)
+FUNC_FUNC_3BUF(bor,  long,  long)
+FUNC_FUNC_3BUF(bor,  unsigned_long,  long)
+
 /* Fortran integer */
 #if OMPI_HAVE_FORTRAN_INTEGER
 FUNC_FUNC_3BUF(bor, fortran_integer, ompi_fortran_integer_t)
@@ -1090,6 +1236,9 @@ FUNC_FUNC_3BUF(bxor,  int32_t,  int32_t)
 FUNC_FUNC_3BUF(bxor, uint32_t, uint32_t)
 FUNC_FUNC_3BUF(bxor,  int64_t,  int64_t)
 FUNC_FUNC_3BUF(bxor, uint64_t, uint64_t)
+FUNC_FUNC_3BUF(bxor,  long,  long)
+FUNC_FUNC_3BUF(bxor,  unsigned_long,  long)
+
 /* Fortran integer */
 #if OMPI_HAVE_FORTRAN_INTEGER
 FUNC_FUNC_3BUF(bxor, fortran_integer, ompi_fortran_integer_t)
@@ -1197,6 +1346,8 @@ LOC_FUNC_3BUF(minloc, long_double_int, <)
   [OMPI_OP_BASE_TYPE_INT32_T] = ompi_op_base_##ftype##_##name##_int32_t,   \
   [OMPI_OP_BASE_TYPE_UINT32_T] = ompi_op_base_##ftype##_##name##_uint32_t, \
   [OMPI_OP_BASE_TYPE_INT64_T] = ompi_op_base_##ftype##_##name##_int64_t,   \
+  [OMPI_OP_BASE_TYPE_LONG] = ompi_op_base_##ftype##_##name##_long,   \
+  [OMPI_OP_BASE_TYPE_UNSIGNED_LONG] = ompi_op_base_##ftype##_##name##_unsigned_long,   \
   [OMPI_OP_BASE_TYPE_UINT64_T] = ompi_op_base_##ftype##_##name##_uint64_t
 
 /** All the Fortran integers ********************************************/
