@@ -231,8 +231,8 @@ static inline void prepare_recv_req_converter(mca_pml_ob1_recv_request_t *req)
                 req->req_recv.req_base.req_addr,
                 0,
                 &req->req_recv.req_base.req_convertor);
-        opal_convertor_get_unpacked_size(&req->req_recv.req_base.req_convertor,
-                                         &req->req_bytes_expected);
+        opal_convertor_get_packed_size(&req->req_recv.req_base.req_convertor,
+                                       &req->req_bytes_expected);
     }
 }
 
@@ -428,9 +428,11 @@ int mca_pml_ob1_recv_request_ack_send_btl(ompi_proc_t* proc,
         mca_bml_base_btl_t* bml_btl, uint64_t hdr_src_req, void *hdr_dst_req,
         uint64_t hdr_rdma_offset, uint64_t size, bool nordma);
 
-static inline int mca_pml_ob1_recv_request_ack_send(ompi_proc_t* proc,
-        uint64_t hdr_src_req, void *hdr_dst_req, uint64_t hdr_send_offset,
-        uint64_t size, bool nordma)
+static inline int
+mca_pml_ob1_recv_request_ack_send(mca_btl_base_module_t* btl,
+                                  ompi_proc_t* proc,
+                                  uint64_t hdr_src_req, void *hdr_dst_req, uint64_t hdr_send_offset,
+                                  uint64_t size, bool nordma)
 {
     size_t i;
     mca_bml_base_btl_t* bml_btl;
@@ -438,11 +440,18 @@ static inline int mca_pml_ob1_recv_request_ack_send(ompi_proc_t* proc,
 
     assert (NULL != endpoint);
 
+    /**
+     * If a btl has been requested then send the ack using that specific device, otherwise
+     * we are free to pick one. We need to force the ack to go over a specific BTL, in order
+     * to prevent the establishement of new connections during the matching handshake.
+     */
     for(i = 0; i < mca_bml_base_btl_array_get_size(&endpoint->btl_eager); i++) {
         bml_btl = mca_bml_base_btl_array_get_next(&endpoint->btl_eager);
-        if(mca_pml_ob1_recv_request_ack_send_btl(proc, bml_btl, hdr_src_req,
-                    hdr_dst_req, hdr_send_offset, size, nordma) == OMPI_SUCCESS)
-            return OMPI_SUCCESS;
+        if( (NULL == btl) || (btl == bml_btl->btl) ) {
+            if(mca_pml_ob1_recv_request_ack_send_btl(proc, bml_btl, hdr_src_req,
+                                                     hdr_dst_req, hdr_send_offset, size, nordma) == OMPI_SUCCESS)
+                return OMPI_SUCCESS;
+        }
     }
 
     MCA_PML_OB1_ADD_ACK_TO_PENDING(proc, hdr_src_req, hdr_dst_req,
