@@ -705,7 +705,7 @@ int mca_pml_ucx_isend_init(const void *buf, size_t count, ompi_datatype_t *datat
     OMPI_DATATYPE_RETAIN(datatype);
 
     if (MCA_PML_BASE_SEND_BUFFERED == mode) {
-        req->datatype = NULL;
+        req->datatype = (ucp_datatype_t)NULL;
     } else {
         req->datatype = mca_pml_ucx_get_datatype(datatype);
     }
