@@ -81,6 +81,11 @@ void mca_pml_ucx_send_completion(void *request, ucs_status_t status)
     mca_pml_ucx_send_completion_internal(request, status);
 }
 
+void mca_pml_ucx_send_completion_empty(void *request, ucs_status_t status)
+{
+    /* empty */
+}
+
 void mca_pml_ucx_bsend_completion(void *request, ucs_status_t status)
 {
     mca_pml_ucx_bsend_completion_internal(request, status);
@@ -212,10 +217,7 @@ static int mca_pml_ucx_persistent_request_free(ompi_request_t **rptr)
         mca_pml_ucx_persistent_request_detach(preq, tmp_req);
         ucp_request_free(tmp_req);
     }
-    if ((preq->flags & MCA_PML_UCX_REQUEST_FLAG_SEND) &&
-         (MCA_PML_BASE_SEND_BUFFERED == preq->send.mode)) {
-        OBJ_RELEASE(preq->datatype.ompi_datatype);
-    }
+    OMPI_DATATYPE_RELEASE(preq->ompi_datatype);
     PML_UCX_FREELIST_RETURN(&ompi_pml_ucx.persistent_reqs, &preq->ompi.super);
     *rptr = MPI_REQUEST_NULL;
     return OMPI_SUCCESS;
