@@ -11,9 +11,9 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
- * Copyright (c) 2016      Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2016-2017 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * $COPYRIGHT$
  *
@@ -42,8 +42,6 @@ static const char FUNC_NAME[] = "MPI_File_get_errhandler";
 
 int MPI_File_get_errhandler( MPI_File file, MPI_Errhandler *errhandler)
 {
-    MPI_Errhandler tmp;
-
     OPAL_CR_NOOP_PROGRESS();
 
   /* Error checking */
@@ -64,16 +62,12 @@ int MPI_File_get_errhandler( MPI_File file, MPI_Errhandler *errhandler)
     }
   }
 
-  /* On 64 bits environments we have to make sure the reading of the
-     error_handler became atomic. */
-  do {
-      tmp = file->error_handler;
-  } while (!OPAL_ATOMIC_CMPSET_PTR(&(file->error_handler), tmp, tmp));
-
+  OPAL_THREAD_LOCK(&file->f_lock);
   /* Retain the errhandler, corresponding to object refcount
      decrease in errhandler_free.c. */
-  *errhandler = tmp;
-  OBJ_RETAIN(tmp);
+  *errhandler = file->error_handler;
+  OBJ_RETAIN(file->error_handler);
+  OPAL_THREAD_UNLOCK(&file->f_lock);
 
   /* All done */
 
