@@ -216,6 +216,9 @@ int ompi_op_init(void)
 #warning Unsupported definition for MPI_COUNT
 #endif
 
+    ompi_op_ddt_map[OMPI_DATATYPE_MPI_LONG] = OMPI_OP_BASE_TYPE_LONG;
+    ompi_op_ddt_map[OMPI_DATATYPE_MPI_UNSIGNED_LONG] = OMPI_OP_BASE_TYPE_UNSIGNED_LONG;
+
     /* Create the intrinsic ops */
 
     if (OMPI_SUCCESS !=
