@@ -88,7 +88,12 @@ const char *ompi_var_type_names[] = {
     "string",
     "version_string",
     "bool",
-    "double"
+    "double",
+    "long",
+    "int32_t",
+    "uint32_t",
+    "int64_t",
+    "uint64_t",
 };
 
 const size_t ompi_var_type_sizes[] = {
@@ -100,7 +105,12 @@ const size_t ompi_var_type_sizes[] = {
     sizeof (char),
     sizeof (char),
     sizeof (bool),
-    sizeof (double)
+    sizeof (double),
+    sizeof (long),
+    sizeof (int32_t),
+    sizeof (uint32_t),
+    sizeof (int64_t),
+    sizeof (uint64_t),
 };
 
 static const char *var_source_names[] = {
@@ -401,7 +411,7 @@ int mca_base_var_cache_files(bool rel_path_search)
 #if OPAL_WANT_HOME_CONFIG_FILES
     asprintf(&mca_base_var_files, "%s"OPAL_PATH_SEP".openmpi" OPAL_PATH_SEP
              "mca-params.conf%c%s" OPAL_PATH_SEP "openmpi-mca-params.conf",
-             home, OPAL_ENV_SEP, opal_install_dirs.sysconfdir);
+             home, ',', opal_install_dirs.sysconfdir);
 #else
     asprintf(&mca_base_var_files, "%s" OPAL_PATH_SEP "openmpi-mca-params.conf",
              opal_install_dirs.sysconfdir);
@@ -517,7 +527,7 @@ int mca_base_var_cache_files(bool rel_path_search)
     if (NULL != mca_base_var_file_prefix) {
        resolve_relative_paths(&mca_base_var_file_prefix, mca_base_param_file_path, rel_path_search, &mca_base_var_files, OPAL_ENV_SEP);
     }
-    read_files (mca_base_var_files, &mca_base_var_file_values, OPAL_ENV_SEP);
+    read_files (mca_base_var_files, &mca_base_var_file_values, ',');
 
     if (NULL != mca_base_envar_file_prefix) {
        resolve_relative_paths(&mca_base_envar_file_prefix, mca_base_param_file_path, rel_path_search, &mca_base_envar_files, ',');
@@ -683,8 +693,13 @@ static int var_set_from_string (mca_base_var_t *var, char *src)
 
     switch (var->mbv_type) {
     case MCA_BASE_VAR_TYPE_INT:
+    case MCA_BASE_VAR_TYPE_INT32_T:
+    case MCA_BASE_VAR_TYPE_UINT32_T:
+    case MCA_BASE_VAR_TYPE_LONG:
     case MCA_BASE_VAR_TYPE_UNSIGNED_INT:
     case MCA_BASE_VAR_TYPE_UNSIGNED_LONG:
+    case MCA_BASE_VAR_TYPE_INT64_T:
+    case MCA_BASE_VAR_TYPE_UINT64_T:
     case MCA_BASE_VAR_TYPE_UNSIGNED_LONG_LONG:
     case MCA_BASE_VAR_TYPE_BOOL:
     case MCA_BASE_VAR_TYPE_SIZE_T:
@@ -710,6 +725,17 @@ static int var_set_from_string (mca_base_var_t *var, char *src)
             MCA_BASE_VAR_TYPE_UNSIGNED_INT == var->mbv_type) {
             int *castme = (int*) var->mbv_storage;
             *castme = int_value;
+        } else if (MCA_BASE_VAR_TYPE_INT32_T == var->mbv_type ||
+            MCA_BASE_VAR_TYPE_UINT32_T == var->mbv_type) {
+            int32_t *castme = (int32_t *) var->mbv_storage;
+            *castme = int_value;
+        } else if (MCA_BASE_VAR_TYPE_INT64_T == var->mbv_type ||
+            MCA_BASE_VAR_TYPE_UINT64_T == var->mbv_type) {
+            int64_t *castme = (int64_t *) var->mbv_storage;
+            *castme = int_value;
+        } else if (MCA_BASE_VAR_TYPE_LONG == var->mbv_type) {
+            long *castme = (long*) var->mbv_storage;
+            *castme = (long) int_value;
         } else if (MCA_BASE_VAR_TYPE_UNSIGNED_LONG == var->mbv_type) {
             unsigned long *castme = (unsigned long*) var->mbv_storage;
             *castme = (unsigned long) int_value;
@@ -1263,11 +1289,18 @@ static int register_variable (const char *project_name, const char *framework_na
     uintptr_t align = 0;
     switch (type) {
     case MCA_BASE_VAR_TYPE_INT:
-        align = OPAL_ALIGNMENT_INT;
-        break;
     case MCA_BASE_VAR_TYPE_UNSIGNED_INT:
         align = OPAL_ALIGNMENT_INT;
         break;
+    case MCA_BASE_VAR_TYPE_INT32_T:
+    case MCA_BASE_VAR_TYPE_UINT32_T:
+        align = OPAL_ALIGNMENT_INT32;
+        break;
+    case MCA_BASE_VAR_TYPE_INT64_T:
+    case MCA_BASE_VAR_TYPE_UINT64_T:
+        align = OPAL_ALIGNMENT_INT64;
+        break;
+    case MCA_BASE_VAR_TYPE_LONG:
     case MCA_BASE_VAR_TYPE_UNSIGNED_LONG:
         align = OPAL_ALIGNMENT_LONG;
         break;
@@ -1896,6 +1929,14 @@ static int var_value_string (mca_base_var_t *var, char **value_string)
 
     assert (MCA_BASE_VAR_TYPE_MAX > var->mbv_type);
 
+    /** Parameters with MCA_BASE_VAR_FLAG_DEF_UNSET flag should be shown
+     * as "unset" by default. */
+    if ((var->mbv_flags & MCA_BASE_VAR_FLAG_DEF_UNSET) &&
+        (MCA_BASE_VAR_SOURCE_DEFAULT == var->mbv_source)){
+        asprintf (value_string, "%s", "unset");
+        return OPAL_SUCCESS;
+    }
+
     ret = mca_base_var_get_value(var->mbv_index, &value, NULL, NULL);
     if (OPAL_SUCCESS != ret || NULL == value) {
         return ret;
@@ -1906,6 +1947,21 @@ static int var_value_string (mca_base_var_t *var, char **value_string)
         case MCA_BASE_VAR_TYPE_INT:
             ret = asprintf (value_string, "%d", value->intval);
             break;
+        case MCA_BASE_VAR_TYPE_INT32_T:
+            ret = asprintf (value_string, "%" PRId32, value->int32tval);
+            break;
+        case MCA_BASE_VAR_TYPE_UINT32_T:
+            ret = asprintf (value_string, "%" PRIu32, value->uint32tval);
+            break;
+        case MCA_BASE_VAR_TYPE_INT64_T:
+            ret = asprintf (value_string, "%" PRId64, value->int64tval);
+            break;
+        case MCA_BASE_VAR_TYPE_UINT64_T:
+            ret = asprintf (value_string, "%" PRIu64, value->uint64tval);
+            break;
+        case MCA_BASE_VAR_TYPE_LONG:
+            ret = asprintf (value_string, "%ld", value->longval);
+            break;
         case MCA_BASE_VAR_TYPE_UNSIGNED_INT:
             ret = asprintf (value_string, "%u", value->uintval);
             break;
