@@ -188,14 +188,14 @@ btl_portals4_init_interface(void)
             goto error;
         }
         OPAL_OUTPUT_VERBOSE((90, opal_btl_base_framework.framework_output, "PtlMEAppend (overflow list) OK for NI %d", interface));
-    }
 
-    ret = mca_btl_portals4_recv_enable(portals4_btl);
-    if (PTL_OK != ret) {
-        opal_output_verbose(1, opal_btl_base_framework.framework_output,
-                            "%s:%d: Initialization of recv buffer failed: %d",
-                            __FILE__, __LINE__, ret);
-        goto error;
+        ret = mca_btl_portals4_recv_enable(portals4_btl);
+        if (PTL_OK != ret) {
+            opal_output_verbose(1, opal_btl_base_framework.framework_output,
+                                "%s:%d: Initialization of recv buffer failed: %d", __FILE__, __LINE__,
+                                ret);
+            goto error;
+        }
     }
 
     return OPAL_SUCCESS;
@@ -418,7 +418,7 @@ mca_btl_portals4_add_procs(struct mca_btl_base_module_t* btl_base,
                               curr_proc,
                               &btl_peer_data[i]);
 
-        OPAL_THREAD_ADD32(&portals4_btl->portals_num_procs, 1);
+        OPAL_THREAD_ADD_FETCH32(&portals4_btl->portals_num_procs, 1);
         /* and here we can reach */
         opal_bitmap_set_bit(reachable, i);
 
@@ -472,7 +472,7 @@ mca_btl_portals4_del_procs(struct mca_btl_base_module_t *btl,
        portals4 entry in proc_endpoints instead of the peer_data */
     for (i = 0 ; i < nprocs ; ++i) {
         free(btl_peer_data[i]);
-        OPAL_THREAD_ADD32(&portals4_btl->portals_num_procs, -1);
+        OPAL_THREAD_ADD_FETCH32(&portals4_btl->portals_num_procs, -1);
     }
 
     return OPAL_SUCCESS;
@@ -530,7 +530,7 @@ mca_btl_portals4_free(struct mca_btl_base_module_t* btl_base,
         if (frag->me_h != PTL_INVALID_HANDLE) {
             frag->me_h = PTL_INVALID_HANDLE;
         }
-        OPAL_THREAD_ADD32(&portals4_btl->portals_outstanding_ops, -1);
+        OPAL_THREAD_ADD_FETCH32(&portals4_btl->portals_outstanding_ops, -1);
         OPAL_OUTPUT_VERBOSE((90, opal_btl_base_framework.framework_output,
             "mca_btl_portals4_free: Decrementing portals_outstanding_ops=%d\n", portals4_btl->portals_outstanding_ops));
         OPAL_BTL_PORTALS4_FRAG_RETURN_USER(portals4_btl, frag);
@@ -558,15 +558,16 @@ mca_btl_portals4_prepare_src(struct mca_btl_base_module_t* btl_base,
                             size_t* size,
                             uint32_t flags)
 {
-    struct mca_btl_portals4_module_t* portals4_btl = (struct mca_btl_portals4_module_t*) btl_base;
-    mca_btl_portals4_frag_t* frag;
+    mca_btl_portals4_frag_t* frag = NULL;
     size_t max_data = *size;
     struct iovec iov;
     uint32_t iov_count = 1;
     int ret;
 
     OPAL_OUTPUT_VERBOSE((90, opal_btl_base_framework.framework_output,
-        "mca_btl_portals4_prepare_src NI=%d reserve=%ld size=%ld max_data=%ld\n", portals4_btl->interface_num, reserve, *size, max_data));
+                         "mca_btl_portals4_prepare_src NI=%d reserve=%ld size=%ld max_data=%ld\n",
+                         ((struct mca_btl_portals4_module_t *) btl_base)->interface_num,
+                         reserve, *size, max_data));
 
     if (0 != reserve || 0 != opal_convertor_need_buffers(convertor)) {
         OPAL_OUTPUT_VERBOSE((90, opal_btl_base_framework.framework_output, "mca_btl_portals4_prepare_src NEED BUFFERS or RESERVE\n"));
@@ -615,7 +616,7 @@ mca_btl_portals4_register_mem(mca_btl_base_module_t *btl_base,
         return NULL;
     }
 
-    handle->key = OPAL_THREAD_ADD64(&(portals4_btl->portals_rdma_key), 1);
+    handle->key = OPAL_THREAD_ADD_FETCH64(&(portals4_btl->portals_rdma_key), 1);
     handle->remote_offset = 0;
 
     OPAL_OUTPUT_VERBOSE((90, opal_btl_base_framework.framework_output,
@@ -655,7 +656,7 @@ mca_btl_portals4_register_mem(mca_btl_base_module_t *btl_base,
         opal_output_verbose(1, opal_btl_base_framework.framework_output,
                             "%s:%d: PtlMEAppend failed: %d\n",
                             __FILE__, __LINE__, ret);
-        OPAL_THREAD_ADD32(&portals4_btl->portals_outstanding_ops, -1);
+        OPAL_THREAD_ADD_FETCH32(&portals4_btl->portals_outstanding_ops, -1);
         return NULL;
     }
     OPAL_OUTPUT_VERBOSE((90, opal_btl_base_framework.framework_output,
@@ -670,11 +671,11 @@ mca_btl_portals4_deregister_mem(mca_btl_base_module_t *btl_base,
                                 mca_btl_base_registration_handle_t *handle)
 {
     int ret;
-    struct mca_btl_portals4_module_t   *portals4_btl = (struct mca_btl_portals4_module_t*) btl_base;
 
     OPAL_OUTPUT_VERBOSE((90, opal_btl_base_framework.framework_output,
-        "mca_btl_portals4_deregister_mem NI=%d handle=%p key=%ld me_h=%d\n",
-        portals4_btl->interface_num, (void *)handle, handle->key, handle->me_h));
+                         "mca_btl_portals4_deregister_mem NI=%d handle=%p key=%ld me_h=%d\n",
+                         ((struct mca_btl_portals4_module_t *) btl_base)->interface_num,
+                         (void *) handle, handle->key, handle->me_h));
 
     if (!PtlHandleIsEqual(handle->me_h, PTL_INVALID_HANDLE)) {
         ret = PtlMEUnlink(handle->me_h);
