@@ -16,7 +16,7 @@
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2015-2016 Los Alamos National Security, LLC. All rights
  *                         reserved.
- * Copyright (c) 2015-2017 Cisco Systems, Inc.  All rights reserved
+ * Copyright (c) 2015-2018 Cisco Systems, Inc.  All rights reserved
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -413,10 +413,10 @@ int mca_btl_tcp_proc_insert( mca_btl_tcp_proc_t* btl_proc,
 {
     struct sockaddr_storage endpoint_addr_ss;
     const char *proc_hostname;
-    unsigned int perm_size;
+    unsigned int perm_size = 0;
     int rc, *a = NULL;
     size_t i, j;
-    mca_btl_tcp_interface_t** peer_interfaces;
+    mca_btl_tcp_interface_t** peer_interfaces = NULL;
     mca_btl_tcp_proc_data_t _proc_data, *proc_data=&_proc_data;
     size_t max_peer_interfaces;
     char str_local[128], str_remote[128];
@@ -455,7 +455,11 @@ int mca_btl_tcp_proc_insert( mca_btl_tcp_proc_t* btl_proc,
 
     max_peer_interfaces = proc_data->max_local_interfaces;
     peer_interfaces = (mca_btl_tcp_interface_t**)calloc( max_peer_interfaces, sizeof(mca_btl_tcp_interface_t*) );
-    assert(NULL != peer_interfaces);
+    if (NULL == peer_interfaces) {
+        max_peer_interfaces = 0;
+        rc = OPAL_ERR_OUT_OF_RESOURCE;
+        goto exit;
+    }
     proc_data->num_peer_interfaces = 0;
 
     /*
@@ -480,8 +484,9 @@ int mca_btl_tcp_proc_insert( mca_btl_tcp_proc_t* btl_proc,
                 max_peer_interfaces <<= 1;
                 peer_interfaces = (mca_btl_tcp_interface_t**)realloc( peer_interfaces,
                                                                       max_peer_interfaces * sizeof(mca_btl_tcp_interface_t*) );
-                if( NULL == peer_interfaces )
+                if( NULL == peer_interfaces ) {
                     return OPAL_ERR_OUT_OF_RESOURCE;
+                }
             }
             peer_interfaces[index] = (mca_btl_tcp_interface_t *) malloc(sizeof(mca_btl_tcp_interface_t));
             mca_btl_tcp_initialise_interface(peer_interfaces[index],
@@ -489,7 +494,7 @@ int mca_btl_tcp_proc_insert( mca_btl_tcp_proc_t* btl_proc,
         }
 
         /*
-         * in case the peer address has all intended connections,
+         * in case the peer address has created all intended connections,
          * mark the complete peer interface as 'not available'
          */
         if(endpoint_addr->addr_inuse >=  mca_btl_tcp_component.tcp_num_links) {
@@ -636,7 +641,8 @@ int mca_btl_tcp_proc_insert( mca_btl_tcp_proc_t* btl_proc,
 
     a = (int *) malloc(perm_size * sizeof(int));
     if (NULL == a) {
-        return OPAL_ERR_OUT_OF_RESOURCE;
+        rc = OPAL_ERR_OUT_OF_RESOURCE;
+        goto exit;
     }
 
     /* Can only find the best set of connections when the number of
@@ -698,6 +704,9 @@ int mca_btl_tcp_proc_insert( mca_btl_tcp_proc_t* btl_proc,
                             OPAL_NAME_PRINT(btl_proc->proc_opal->proc_name));
     }
 
+ exit:
+    // Ok to always free because proc_data() was memset() to 0 before
+    // any possible return (and free(NULL) is fine).
     for(i = 0; i < perm_size; ++i) {
         free(proc_data->weights[i]);
         free(proc_data->best_addr[i]);
@@ -723,12 +732,12 @@ int mca_btl_tcp_proc_insert( mca_btl_tcp_proc_t* btl_proc,
         }
         free(proc_data->local_interfaces[i]);
     }
-    free(proc_data->local_interfaces);
+    free(proc_data->local_interfaces); proc_data->local_interfaces = NULL;
     proc_data->max_local_interfaces = 0;
 
-    free(proc_data->weights);
-    free(proc_data->best_addr);
-    free(proc_data->best_assignment);
+    free(proc_data->weights); proc_data->weights = NULL;
+    free(proc_data->best_addr); proc_data->best_addr = NULL;
+    free(proc_data->best_assignment); proc_data->best_assignment = NULL;
 
     OBJ_DESTRUCT(&_proc_data.local_kindex_to_index);
     OBJ_DESTRUCT(&_proc_data.peer_kindex_to_index);
@@ -817,12 +826,15 @@ mca_btl_tcp_proc_t* mca_btl_tcp_proc_lookup(const opal_process_name_t *name)
 void mca_btl_tcp_proc_accept(mca_btl_tcp_proc_t* btl_proc, struct sockaddr* addr, int sd)
 {
     OPAL_THREAD_LOCK(&btl_proc->proc_lock);
+    int found_match = 0;
+    mca_btl_base_endpoint_t* match_btl_endpoint;
+
     for( size_t i = 0; i < btl_proc->proc_endpoint_count; i++ ) {
         mca_btl_base_endpoint_t* btl_endpoint = btl_proc->proc_endpoints[i];
         /* We are not here to make a decision about what is good socket
          * and what is not. We simply check that this socket fit the endpoint
          * end we prepare for the real decision function mca_btl_tcp_endpoint_accept. */
-        if( btl_endpoint->endpoint_addr->addr_family != addr->sa_family ) {
+        if( btl_endpoint->endpoint_addr->addr_family != addr->sa_family) {
             continue;
         }
         switch (addr->sa_family) {
@@ -840,6 +852,10 @@ void mca_btl_tcp_proc_accept(mca_btl_tcp_proc_t* btl_proc, struct sockaddr* addr
                                               tmp[1], 16),
                                     (int)i, (int)btl_proc->proc_endpoint_count);
                 continue;
+            } else if (btl_endpoint->endpoint_state != MCA_BTL_TCP_CLOSED) {
+                 found_match = 1;
+                 match_btl_endpoint = btl_endpoint;
+                 continue;
             }
             break;
 #if OPAL_ENABLE_IPV6
@@ -857,6 +873,10 @@ void mca_btl_tcp_proc_accept(mca_btl_tcp_proc_t* btl_proc, struct sockaddr* addr
                                               tmp[1], INET6_ADDRSTRLEN),
                                     (int)i, (int)btl_proc->proc_endpoint_count);
                 continue;
+            } else if (btl_endpoint->endpoint_state != MCA_BTL_TCP_CLOSED) {
+                 found_match = 1;
+                 match_btl_endpoint = btl_endpoint;
+                 continue;
             }
             break;
 #endif
@@ -864,10 +884,20 @@ void mca_btl_tcp_proc_accept(mca_btl_tcp_proc_t* btl_proc, struct sockaddr* addr
             ;
         }
 
+        /* Set state to CONNECTING to ensure that subsequent conenctions do not attempt to re-use endpoint in the num_links > 1 case*/
+        btl_endpoint->endpoint_state = MCA_BTL_TCP_CONNECTING;
         (void)mca_btl_tcp_endpoint_accept(btl_endpoint, addr, sd);
         OPAL_THREAD_UNLOCK(&btl_proc->proc_lock);
         return;
     }
+    /* In this case the connection was inbound to an address exported, but was not in a CLOSED state.
+     * mca_btl_tcp_endpoint_accept() has logic to deal with the race condition that has likely caused this
+     * scenario, so call it here.*/
+    if (found_match) {
+        (void)mca_btl_tcp_endpoint_accept(match_btl_endpoint, addr, sd);
+        OPAL_THREAD_UNLOCK(&btl_proc->proc_lock);
+        return;
+    }
     /* No further use of this socket. Close it */
     CLOSE_THE_SOCKET(sd);
     {
