@@ -4,7 +4,12 @@
  *                         reserved.
  * Copyright (c) 2020      Triad National Security, LLC. All rights
  *                         reserved.
- * Copyright (c) 2020      Cisco Systems, Inc.  All rights reserved
+ * Copyright (c) 2021      Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
+ * Copyright (c) 2020-2021 Cisco Systems, Inc.  All rights reserved
+ * Copyright (c) 2021      Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021      Amazon.com, Inc. or its affiliates. All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -13,10 +18,16 @@
  */
 
 
+#include "opal_config.h"
+
 #include <errno.h>
 #include <unistd.h>
+#include <rdma/fabric.h>
+#include <rdma/fi_errno.h>
+#ifdef HAVE_RDMA_FI_EXT_H
+#include <rdma/fi_ext.h>
+#endif
 
-#include "opal_config.h"
 #include "common_ofi.h"
 #include "opal_config.h"
 #include "opal/constants.h"
@@ -24,19 +35,220 @@
 #include "opal/mca/base/mca_base_var.h"
 #include "opal/mca/base/mca_base_framework.h"
 #include "opal/mca/hwloc/base/base.h"
+#include "opal/mca/memory/base/base.h"
 #include "opal/mca/pmix/base/base.h"
 #include "opal/util/show_help.h"
 
-OPAL_DECLSPEC opal_common_ofi_module_t opal_common_ofi = {
-    .prov_include = NULL,
-    .prov_exclude = NULL,
-    .registered = 0,
-    .verbose = 0
+opal_common_ofi_module_t opal_common_ofi = {.prov_include = NULL,
+                                            .prov_exclude = NULL,
+                                            .output = -1};
+static const char default_prov_exclude_list[] = "shm,sockets,tcp,udp,rstream,usnic";
+static opal_mutex_t opal_common_ofi_mutex = OPAL_MUTEX_STATIC_INIT;
+static int opal_common_ofi_verbose_level = 0;
+static int opal_common_ofi_init_ref_cnt = 0;
+#ifdef HAVE_STRUCT_FI_OPS_MEM_MONITOR
+static bool opal_common_ofi_installed_memory_monitor = false;
+#endif
+
+#ifdef HAVE_STRUCT_FI_OPS_MEM_MONITOR
+
+/*
+ * Monitor object to export into Libfabric to provide memory release
+ * notifications using our own memory hooks framework.  Monitors may
+ * use the subscribe/unsubscribe notifications to reduce unnecessary
+ * notifications, but are not required to do so.  Because patcher
+ * notifies about all releases, it is cheaper for us to not filter and
+ * this monitor can safely ignore subscribe/unsubscribe notifications.
+ *
+ * Libfabric requires the object to be fully defined.  Unlike most of
+ * Open MPI, it does not have NULL function pointer checks in calling
+ * code.
+ */
+static int opal_common_ofi_monitor_start(struct fid_mem_monitor *monitor)
+{
+    return 0;
+}
+
+static void opal_common_ofi_monitor_stop(struct fid_mem_monitor *monitor)
+{
+    return;
+}
+
+static int opal_common_ofi_monitor_subscribe(struct fid_mem_monitor *monitor,
+                                             const void *addr, size_t len)
+{
+    return 0;
+}
+
+static void opal_common_ofi_monitor_unsubscribe(struct fid_mem_monitor *monitor,
+                                                const void *addr, size_t len)
+{
+    return;
+}
+
+static bool opal_common_ofi_monitor_valid(struct fid_mem_monitor *monitor,
+                                     const void *addr, size_t len)
+{
+    return true;
+}
+
+static struct fid_mem_monitor *opal_common_ofi_monitor = NULL;
+static struct fid *opal_common_ofi_cache_fid = NULL;
+static struct fi_ops_mem_monitor opal_common_ofi_export_ops = {
+    .size = sizeof(struct fi_ops_mem_monitor),
+    .start = opal_common_ofi_monitor_start,
+    .stop = opal_common_ofi_monitor_stop,
+    .subscribe = opal_common_ofi_monitor_subscribe,
+    .unsubscribe = opal_common_ofi_monitor_unsubscribe,
+    .valid = opal_common_ofi_monitor_valid,
 };
 
-static const char default_prov_exclude_list[] = "shm,sockets,tcp,udp,rstream";
+/**
+ * Callback function from Open MPI memory monitor
+ *
+ * Translation function between the callback function from Open MPI's
+ * memory notifier to the Libfabric memory monitor.
+ */
+static void opal_common_ofi_mem_release_cb(void *buf, size_t length,
+                                           void *cbdata, bool from_alloc)
+{
+    opal_common_ofi_monitor->import_ops->notify(opal_common_ofi_monitor,
+                                                buf, length);
+}
 
-OPAL_DECLSPEC int opal_common_ofi_is_in_list(char **list, char *item)
+#endif /* HAVE_STRUCT_FI_OPS_MEM_MONITOR */
+
+int opal_common_ofi_export_memory_monitor(void)
+{
+    int ret = -FI_ENOSYS;
+
+#ifdef HAVE_STRUCT_FI_OPS_MEM_MONITOR
+    OPAL_THREAD_LOCK(&opal_common_ofi_mutex);
+
+    if (NULL != opal_common_ofi_cache_fid) {
+        return 0;
+    }
+
+    /*
+     * While the memory import functionality was introduced in 1.13,
+     * some deadlock bugs exist in the 1.13 series.  Require version
+     * 1.14 before this code is activated.  Not activating the code
+     * should not break any functionality directly, but may lead to
+     * sub-optimal memory monitors being used in Libfabric, as Open
+     * MPI will almost certainly install a patcher first.
+     */
+    if (FI_VERSION_LT(fi_version(), FI_VERSION(1, 14))) {
+        ret = -FI_ENOSYS;
+        goto err;
+    }
+
+    ret = mca_base_framework_open(&opal_memory_base_framework, 0);
+    if (OPAL_SUCCESS != ret) {
+        ret = -FI_ENOSYS;
+        goto err;
+    }
+    if ((OPAL_MEMORY_FREE_SUPPORT | OPAL_MEMORY_MUNMAP_SUPPORT)
+        != (((OPAL_MEMORY_FREE_SUPPORT | OPAL_MEMORY_MUNMAP_SUPPORT))
+        & opal_mem_hooks_support_level())) {
+        ret = -FI_ENOSYS;
+        goto err;
+    }
+
+    /*
+     * The monitor import object has the well known name "mr_cache"
+     * and was introduced in Libfabric 1.13
+     */
+    ret = fi_open(FI_VERSION(1,13), "mr_cache", NULL, 0, 0,
+                  &opal_common_ofi_cache_fid, NULL);
+    if (0 != ret) {
+        goto err;
+    }
+
+    opal_common_ofi_monitor = calloc(1, sizeof(*opal_common_ofi_monitor));
+    if (NULL == opal_common_ofi_monitor) {
+        ret = -FI_ENOMEM;
+        goto err;
+    }
+
+    opal_common_ofi_monitor->fid.fclass = FI_CLASS_MEM_MONITOR;
+    opal_common_ofi_monitor->export_ops = &opal_common_ofi_export_ops;
+    ret = fi_import_fid(opal_common_ofi_cache_fid,
+                        &opal_common_ofi_monitor->fid, 0);
+    if (0 != ret) {
+        goto err;
+    }
+    opal_mem_hooks_register_release(opal_common_ofi_mem_release_cb, NULL);
+    opal_common_ofi_installed_memory_monitor = true;
+
+    ret = 0;
+
+err:
+    if (0 != ret) {
+        if (NULL != opal_common_ofi_cache_fid) {
+            fi_close(opal_common_ofi_cache_fid);
+        }
+        if (NULL != opal_common_ofi_monitor) {
+            free(opal_common_ofi_monitor);
+        }
+
+        opal_common_ofi_installed_memory_monitor = false;
+    }
+
+    OPAL_THREAD_UNLOCK(&opal_common_ofi_mutex);
+#endif
+
+    return ret;
+}
+
+static int opal_common_ofi_remove_memory_monitor(void)
+{
+#ifdef HAVE_STRUCT_FI_OPS_MEM_MONITOR
+    if (opal_common_ofi_installed_memory_monitor) {
+        opal_mem_hooks_unregister_release(opal_common_ofi_mem_release_cb);
+        fi_close(opal_common_ofi_cache_fid);
+        fi_close(&opal_common_ofi_monitor->fid);
+        free(opal_common_ofi_monitor);
+        opal_common_ofi_installed_memory_monitor = false;
+    }
+#endif
+
+    return OPAL_SUCCESS;
+}
+
+int opal_common_ofi_open(void)
+{
+    if ((opal_common_ofi_init_ref_cnt++) > 0) {
+        return OPAL_SUCCESS;
+    }
+
+    return OPAL_SUCCESS;
+}
+
+int opal_common_ofi_close(void)
+{
+    int ret;
+
+    if ((--opal_common_ofi_init_ref_cnt) > 0) {
+        return OPAL_SUCCESS;
+    }
+
+    ret = opal_common_ofi_remove_memory_monitor();
+    if (OPAL_SUCCESS != ret) {
+        return ret;
+    }
+
+    if (-1 != opal_common_ofi.output) {
+        opal_output_close(opal_common_ofi.output);
+        opal_common_ofi.output = -1;
+        if (OPAL_SUCCESS != ret) {
+            return ret;
+        }
+    }
+
+    return OPAL_SUCCESS;
+}
+
+int opal_common_ofi_is_in_list(char **list, char *item)
 {
     int i = 0;
 
@@ -55,89 +267,123 @@ OPAL_DECLSPEC int opal_common_ofi_is_in_list(char **list, char *item)
     return 0;
 }
 
-OPAL_DECLSPEC int opal_common_ofi_register_mca_variables(const mca_base_component_t *component)
+int opal_common_ofi_mca_register(const mca_base_component_t *component)
 {
-    static int registered = 0;
-    static int include_index;
-    static int exclude_index;
-    static int verbose_index;
+    static int include_index = -1;
+    static int exclude_index = -1;
+    static int verbose_index = -1;
+    int ret;
 
     if (fi_version() < FI_VERSION(1,0)) {
         return OPAL_ERROR;
     }
 
-    if (!registered) {
+    OPAL_THREAD_LOCK(&opal_common_ofi_mutex);
+
+    if (0 > include_index) {
         /*
          * this monkey business is needed because of the way the MCA VARs stuff tries to handle pointers to strings when
          * when destructing the MCA var database.  If you don't do something like this,the MCA var framework will try
          * to dereference a pointer which itself is no longer a valid address owing to having been previously dlclosed.
          */
-         opal_common_ofi.prov_include = (char **)malloc(sizeof(char *));
-         *opal_common_ofi.prov_include = NULL;
-         include_index = mca_base_var_register("opal", "opal_common", "ofi",
-                               "provider_include",
-                               "Comma-delimited list of OFI providers that are considered for use (e.g., \"psm,psm2\"; an empty value means that all providers will be considered). Mutually exclusive with mtl_ofi_provider_exclude.",
-                               MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
-                               OPAL_INFO_LVL_1,
-                               MCA_BASE_VAR_SCOPE_READONLY,
-                               opal_common_ofi.prov_include);
-        opal_common_ofi.prov_exclude = (char **)malloc(sizeof(char *));
+        if (NULL == opal_common_ofi.prov_include) {
+            opal_common_ofi.prov_include = (char **) malloc(sizeof(char *));
+            assert(NULL != opal_common_ofi.prov_include);
+        }
+        *opal_common_ofi.prov_include = NULL;
+        include_index = mca_base_var_register(
+            "opal", "opal_common", "ofi", "provider_include",
+            "Comma-delimited list of OFI providers that are considered for use (e.g., "
+            "\"psm,psm2\"; an empty value means that all providers will be considered). Mutually "
+            "exclusive with mtl_ofi_provider_exclude.",
+            MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_1, MCA_BASE_VAR_SCOPE_READONLY,
+            opal_common_ofi.prov_include);
+        if (0 > include_index) {
+            ret = include_index;
+            goto err;
+        }
+    }
+
+    if (0 > exclude_index) {
+        if (NULL == opal_common_ofi.prov_exclude) {
+            opal_common_ofi.prov_exclude = (char **) malloc(sizeof(char *));
+            assert(NULL != opal_common_ofi.prov_exclude);
+        }
         *opal_common_ofi.prov_exclude = strdup(default_prov_exclude_list);
-        exclude_index = mca_base_var_register("opal", "opal_common", "ofi",
-                              "provider_exclude",
-                              "Comma-delimited list of OFI providers that are not considered for use (default: \"sockets,mxm\"; empty value means that all providers will be considered). Mutually exclusive with mtl_ofi_provider_include.",
-                              MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
-                              OPAL_INFO_LVL_1,
-                              MCA_BASE_VAR_SCOPE_READONLY,
-                              opal_common_ofi.prov_exclude);
+        exclude_index = mca_base_var_register(
+            "opal", "opal_common", "ofi", "provider_exclude",
+            "Comma-delimited list of OFI providers that are not considered for use (default: "
+            "\"sockets,mxm\"; empty value means that all providers will be considered). Mutually "
+            "exclusive with mtl_ofi_provider_include.",
+            MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_1, MCA_BASE_VAR_SCOPE_READONLY,
+            opal_common_ofi.prov_exclude);
+        if (0 > exclude_index) {
+            ret = exclude_index;
+            goto err;
+        }
+    }
+
+    if (0 > verbose_index) {
         verbose_index = mca_base_var_register("opal", "opal_common", "ofi", "verbose",
                                               "Verbose level of the OFI components",
                                               MCA_BASE_VAR_TYPE_INT, NULL, 0,
                                               MCA_BASE_VAR_FLAG_SETTABLE, OPAL_INFO_LVL_3,
                                               MCA_BASE_VAR_SCOPE_LOCAL,
-                                              &opal_common_ofi.verbose);
-        registered = 1;
+                                              &opal_common_ofi_verbose_level);
+        if (0 > verbose_index) {
+            ret = verbose_index;
+            goto err;
+        }
     }
 
     if (component) {
-        mca_base_var_register_synonym(include_index, component->mca_project_name,
-                                      component->mca_type_name,
-                                      component->mca_component_name,
-                                      "provider_include", 0);
-        mca_base_var_register_synonym(exclude_index, component->mca_project_name,
-                                      component->mca_type_name,
-                                      component->mca_component_name,
-                                      "provider_exclude", 0);
-        mca_base_var_register_synonym(verbose_index, component->mca_project_name,
-                                      component->mca_type_name,
-                                      component->mca_component_name,
-                                      "verbose", 0);
+        ret = mca_base_var_register_synonym(include_index,
+                                            component->mca_project_name,
+                                            component->mca_type_name,
+                                            component->mca_component_name,
+                                            "provider_include", 0);
+        if (0 > ret) {
+            goto err;
+        }
+        ret = mca_base_var_register_synonym(exclude_index,
+                                            component->mca_project_name,
+                                            component->mca_type_name,
+                                            component->mca_component_name,
+                                            "provider_exclude", 0);
+        if (0 > ret) {
+            goto err;
+        }
+        ret = mca_base_var_register_synonym(verbose_index,
+                                            component->mca_project_name,
+                                            component->mca_type_name,
+                                            component->mca_component_name,
+                                            "verbose", 0);
+        if (0 > ret) {
+            goto err;
+        }
     }
 
-    return OPAL_SUCCESS;
-}
-
-OPAL_DECLSPEC void opal_common_ofi_mca_register(void)
-{
-    opal_common_ofi.registered++;
-    if (opal_common_ofi.registered > 1) {
-         opal_output_set_verbosity(opal_common_ofi.output, opal_common_ofi.verbose);
-        return;
+    /* The frameworks initialize their output streams during
+     * register(), so we similarly try to initialize the output stream
+     * as early as possible.  Because we may register synonyms for
+     * each dependent component, we don't necessarily have all the
+     * data to set verbosity during the first call to
+     * common_ofi_register().  The MCA infrastructure has rules on
+     * synonym value evaluation, so our rubric is to re-set verbosity
+     * after every call to register() (which has registered a new
+     * synonym).  This is not perfect, but it's not horrible, either.
+     */
+    if (opal_common_ofi.output == -1) {
+        opal_common_ofi.output = opal_output_open(NULL);
     }
+    opal_output_set_verbosity(opal_common_ofi.output, opal_common_ofi_verbose_level);
 
-    opal_common_ofi.output = opal_output_open(NULL);
-    opal_output_set_verbosity(opal_common_ofi.output, opal_common_ofi.verbose);
-}
+    ret = OPAL_SUCCESS;
 
-OPAL_DECLSPEC void opal_common_ofi_mca_deregister(void)
-{
-    /* unregister only on last deregister */
-    opal_common_ofi.registered--;
-    assert(opal_common_ofi.registered >= 0);
-    if (opal_common_ofi.registered) {
-        return;
-    }
-    opal_output_close(opal_common_ofi.output);
+err:
+    OPAL_THREAD_UNLOCK(&opal_common_ofi_mutex);
+
+    return ret;
 }
 
 /* check that the tx attributes match */
@@ -190,6 +436,7 @@ check_provider_attr(struct fi_info *provider_info,
 {
     /* make sure both info are the same provider and provide the same attributes */
     if (0 == strcmp(provider_info->fabric_attr->prov_name, provider->fabric_attr->prov_name) &&
+        0 == strcmp(provider_info->fabric_attr->name, provider->fabric_attr->name) &&
         !check_tx_attr(provider_info->tx_attr, provider->tx_attr) &&
         !check_rx_attr(provider_info->rx_attr, provider->rx_attr) &&
         !check_ep_attr(provider_info->ep_attr, provider->ep_attr) &&
@@ -302,26 +549,24 @@ static uint32_t get_package_rank(int32_t num_local_peers, uint16_t my_local_rank
     int i;
     uint16_t relative_locality;
     uint16_t current_package_rank = 0;
-    uint16_t package_ranks[num_local_peers];
+    uint16_t package_ranks[num_local_peers + 1];
     opal_process_name_t pname;
     opal_status_t rc;
     char **peers = NULL;
     char *local_peers = NULL;
     char *locality_string = NULL;
     char *mylocality = NULL;
+    uint16_t *package_rank_ptr;
 
     pname.jobid = OPAL_PROC_MY_NAME.jobid;
     pname.vpid = OPAL_VPID_WILDCARD;
 
-#if HAVE_DECL_PMIX_PACKAGE_RANK
-    uint16_t *package_rank_ptr;
     // Try to get the PACKAGE_RANK from PMIx
     OPAL_MODEX_RECV_VALUE_OPTIONAL(rc, OPAL_PMIX_PACKAGE_RANK,
                                    &pname, &package_rank_ptr, OPAL_UINT16);
     if (OPAL_SUCCESS == rc) {
         return (uint32_t)*package_rank_ptr;
     }
-#endif
 
     // Get the local peers
     OPAL_MODEX_RECV_VALUE(rc, OPAL_PMIX_LOCAL_PEERS,
@@ -356,7 +601,10 @@ static uint32_t get_package_rank(int32_t num_local_peers, uint16_t my_local_rank
                                        &pname, &locality_string, OPAL_STRING);
         if (OPAL_SUCCESS != rc || NULL == locality_string) {
             // If we don't have information about locality, fall back to procid
-            opal_show_help("help-common-ofi.txt", "package_rank failed", true);
+            int level = 10;
+            if (opal_output_get_verbosity(opal_common_ofi.output) >= level) {
+                opal_show_help("help-common-ofi.txt", "package_rank failed", true, level);
+            }
             return pid;
         }
 
@@ -374,61 +622,6 @@ static uint32_t get_package_rank(int32_t num_local_peers, uint16_t my_local_rank
     return (uint32_t)package_ranks[my_local_rank];
 }
 
-/* Selects a NIC based on hardware locality between process cpuset and device BDF.
- *
- * Initializes opal_hwloc_topology to access hardware topology if not previously
- * initialized
- *
- * There are 3 main cases that this covers:
- *
- *      1. If the first provider passed into this function is the only valid
- *      provider, this provider is returned.
- *
- *      2. If there is more than 1 provider that matches the type of the first
- *      provider in the list, and the BDF data
- *      is available then a provider is selected based on locality of device
- *      cpuset and process cpuset and tries to ensure that processes are distributed
- *      evenly across NICs. This has two separate cases:
- *
- *          i. There is one or more provider local to the process:
- *
- *              (local rank % number of providers of the same type that share the process cpuset)
- *              is used to select one of these providers.
- *
- *          ii. There is no provider that is local to the process:
- *
- *              (local rank % number of providers of the same type)
- *              is used to select one of these providers
- *
- *      3. If there is more than 1 providers of the same type in the list, and the BDF data
- *      is not available (the ofi version does not support fi_info.nic or the
- *      provider does not support BDF) then (local rank % number of providers of the same type)
- *      is used to select one of these providers
- *
- *      @param provider_list (IN)   struct fi_info* An initially selected
- *                                  provider NIC. The provider name and
- *                                  attributes are used to restrict NIC
- *                                  selection. This provider is returned if the
- *                                  NIC selection fails.
- *
- *      @param package_rank (IN)   uint32_t The rank of the process. Used to
- *                                  select one valid NIC if there is a case
- *                                  where more than one can be selected. This
- *                                  could occur when more than one provider
- *                                  shares the same cpuset as the process.
- *                                  This could either be a package_rank if one is
- *                                  successfully calculated, or the process id.
- *
- *      @param provider (OUT)       struct fi_info* object with the selected
- *                                  provider if the selection succeeds
- *                                  if the selection fails, returns the fi_info
- *                                  object that was initially provided.
- *
- * All errors should be recoverable and will return the initially provided
- * provider. However, if an error occurs we can no longer guarantee
- * that the provider returned is local to the process or that the processes will
- * balance across available NICs.
- */
 struct fi_info*
 opal_mca_common_ofi_select_provider(struct fi_info *provider_list, int32_t num_local_peers,
                                     uint16_t my_local_rank, char *cpuset, uint32_t pid)
@@ -470,7 +663,9 @@ opal_mca_common_ofi_select_provider(struct fi_info *provider_list, int32_t num_l
         if (!check_provider_attr(provider, current_provider)) {
             cpusets_match = false;
 #if OPAL_OFI_PCI_DATA_AVAILABLE
-            if (NULL != current_provider->nic) {
+            if (NULL != current_provider->nic
+                && NULL != current_provider->nic->bus_attr
+                && current_provider->nic->bus_attr->bus_type == FI_BUS_PCI) {
                 pci = current_provider->nic->bus_attr->attr.pci;
                 cpusets_match = compare_cpusets(opal_hwloc_topology, pci);
             }
@@ -505,7 +700,9 @@ opal_mca_common_ofi_select_provider(struct fi_info *provider_list, int32_t num_l
     }
 
 #if OPAL_OFI_PCI_DATA_AVAILABLE
-    if (NULL != provider->nic) {
+    if (NULL != provider->nic
+        && NULL != provider->nic->bus_attr
+        && provider->nic->bus_attr->bus_type == FI_BUS_PCI) {
         pci = provider->nic->bus_attr->attr.pci;
         cpusets_match = compare_cpusets(opal_hwloc_topology, pci);
     }
