@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2017 Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2018 Intel, Inc. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -243,6 +243,18 @@ typedef int (*opal_pmix_server_job_control_fn_t)(const opal_process_name_t *requ
 
 /* we do not provide a monitoring capability */
 
+/* Request forwarding of specified IO channels to the local PMIx server
+ * for distribution to local clients */
+typedef int (*opal_pmix_server_iof_pull_fn_t)(opal_list_t *sources,
+                                              opal_list_t *directives,
+                                              opal_pmix_iof_channel_t channels,
+                                              opal_pmix_op_cbfunc_t cbfunc, void *cbdata);
+
+/* Entry point for pushing forwarded IO to clients/tools */
+typedef int (*opal_pmix_server_iof_push_fn_t)(const opal_process_name_t *source,
+                                              opal_pmix_iof_channel_t channel,
+                                              unsigned char *data, size_t nbytes);
+
 typedef struct opal_pmix_server_module_1_0_0_t {
     opal_pmix_server_client_connected_fn_t      client_connected;
     opal_pmix_server_client_finalized_fn_t      client_finalized;
@@ -264,6 +276,8 @@ typedef struct opal_pmix_server_module_1_0_0_t {
     opal_pmix_server_listener_fn_t              listener;
     opal_pmix_server_alloc_fn_t                 allocate;
     opal_pmix_server_job_control_fn_t           job_control;
+    opal_pmix_server_iof_pull_fn_t              iof_pull;
+    opal_pmix_server_iof_push_fn_t              iof_push;
 } opal_pmix_server_module_t;
 
 
