@@ -12,7 +12,7 @@
  *                         All rights reserved.
  * Copyright (c) 2010      Oracle and/or its affiliates.  All rights reserved.
  * Copyright (c) 2011      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2013-2017 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2013-2018 Intel, Inc.  All rights reserved.
  * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2016-2017 Research Organization for Information Science
@@ -39,9 +39,11 @@
 #include <errno.h>
 
 #include "opal/hash_string.h"
+#include "opal/util/arch.h"
 #include "opal/util/argv.h"
 #include "opal/util/opal_environ.h"
 #include "opal/util/path.h"
+#include "opal/util/timings.h"
 #include "opal/runtime/opal_progress_threads.h"
 #include "opal/mca/installdirs/installdirs.h"
 #include "opal/mca/pmix/base/base.h"
@@ -49,8 +51,11 @@
 
 #include "orte/util/show_help.h"
 #include "orte/util/proc_info.h"
-#include "orte/mca/errmgr/errmgr.h"
+#include "orte/mca/errmgr/base/base.h"
+#include "orte/mca/filem/base/base.h"
 #include "orte/mca/plm/base/base.h"
+#include "orte/mca/rml/base/rml_contact.h"
+#include "orte/mca/state/base/base.h"
 #include "orte/util/name_fns.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/util/session_dir.h"
@@ -63,11 +68,12 @@
 
 static int rte_init(void);
 static int rte_finalize(void);
+static void rte_abort(int status, bool report);
 
 orte_ess_base_module_t orte_ess_singleton_module = {
     rte_init,
     rte_finalize,
-    orte_ess_base_app_abort,
+    rte_abort,
     NULL /* ft_event */
 };
 
@@ -272,15 +278,190 @@ static int rte_init(void)
         }
     }
 
-    /* use the std app init to complete the procedure */
-    if (ORTE_SUCCESS != (rc = orte_ess_base_app_setup(true))) {
-        ORTE_ERROR_LOG(rc);
-        return rc;
+    /* now that we have all required info, complete the setup */
+    /*
+     * stdout/stderr buffering
+     * If the user requested to override the default setting then do
+     * as they wish.
+     */
+    if( orte_ess_base_std_buffering > -1 ) {
+        if( 0 == orte_ess_base_std_buffering ) {
+            setvbuf(stdout, NULL, _IONBF, 0);
+            setvbuf(stderr, NULL, _IONBF, 0);
+        }
+        else if( 1 == orte_ess_base_std_buffering ) {
+            setvbuf(stdout, NULL, _IOLBF, 0);
+            setvbuf(stderr, NULL, _IOLBF, 0);
+        }
+        else if( 2 == orte_ess_base_std_buffering ) {
+            setvbuf(stdout, NULL, _IOFBF, 0);
+            setvbuf(stderr, NULL, _IOFBF, 0);
+        }
+    }
+
+    /* if I am an MPI app, we will let the MPI layer define and
+     * control the opal_proc_t structure. Otherwise, we need to
+     * do so here */
+    if (ORTE_PROC_NON_MPI) {
+        orte_process_info.super.proc_name = *(opal_process_name_t*)ORTE_PROC_MY_NAME;
+        orte_process_info.super.proc_hostname = orte_process_info.nodename;
+        orte_process_info.super.proc_flags = OPAL_PROC_ALL_LOCAL;
+        orte_process_info.super.proc_arch = opal_local_arch;
+        opal_proc_local_set(&orte_process_info.super);
+    }
+
+    /* open and setup the state machine */
+    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_state_base_framework, 0))) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_state_base_open";
+        goto error;
+    }
+    if (ORTE_SUCCESS != (ret = orte_state_base_select())) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_state_base_select";
+        goto error;
+    }
+
+    /* open the errmgr */
+    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_errmgr_base_framework, 0))) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_errmgr_base_open";
+        goto error;
+    }
+
+    /* setup my session directory */
+    if (orte_create_session_dirs) {
+        OPAL_OUTPUT_VERBOSE((2, orte_ess_base_framework.framework_output,
+                             "%s setting up session dir with\n\ttmpdir: %s\n\thost %s",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                             (NULL == orte_process_info.tmpdir_base) ? "UNDEF" : orte_process_info.tmpdir_base,
+                             orte_process_info.nodename));
+        if (ORTE_SUCCESS != (ret = orte_session_dir(true, ORTE_PROC_MY_NAME))) {
+            ORTE_ERROR_LOG(ret);
+            error = "orte_session_dir";
+            goto error;
+        }
+        /* Once the session directory location has been established, set
+           the opal_output env file location to be in the
+           proc-specific session directory. */
+        opal_output_set_output_file_info(orte_process_info.proc_session_dir,
+                                         "output-", NULL, NULL);
+        /* register the directory for cleanup */
+        if (NULL != opal_pmix.register_cleanup) {
+            if (orte_standalone_operation) {
+                if (OPAL_SUCCESS != (ret = opal_pmix.register_cleanup(orte_process_info.top_session_dir, true, false, true))) {
+                    ORTE_ERROR_LOG(ret);
+                    error = "register cleanup";
+                    goto error;
+                }
+            } else {
+                if (OPAL_SUCCESS != (ret = opal_pmix.register_cleanup(orte_process_info.job_session_dir, true, false, false))) {
+                    ORTE_ERROR_LOG(ret);
+                    error = "register cleanup";
+                    goto error;
+                }
+            }
+        }
+    }
+
+    /* if we have info on the HNP and local daemon, process it */
+    if (NULL != orte_process_info.my_hnp_uri) {
+        /* we have to set the HNP's name, even though we won't route messages directly
+         * to it. This is required to ensure that we -do- send messages to the correct
+         * HNP name
+         */
+        if (ORTE_SUCCESS != (ret = orte_rml_base_parse_uris(orte_process_info.my_hnp_uri,
+                                                            ORTE_PROC_MY_HNP, NULL))) {
+            ORTE_ERROR_LOG(ret);
+            error = "orte_rml_parse_HNP";
+            goto error;
+        }
+    }
+    if (NULL != orte_process_info.my_daemon_uri) {
+        opal_value_t val;
+
+        /* extract the daemon's name so we can update the routing table */
+        if (ORTE_SUCCESS != (ret = orte_rml_base_parse_uris(orte_process_info.my_daemon_uri,
+                                                            ORTE_PROC_MY_DAEMON, NULL))) {
+            ORTE_ERROR_LOG(ret);
+            error = "orte_rml_parse_daemon";
+            goto error;
+        }
+        /* Set the contact info in the database - this won't actually establish
+         * the connection, but just tells us how to reach the daemon
+         * if/when we attempt to send to it
+         */
+        OBJ_CONSTRUCT(&val, opal_value_t);
+        val.key = OPAL_PMIX_PROC_URI;
+        val.type = OPAL_STRING;
+        val.data.string = orte_process_info.my_daemon_uri;
+        if (OPAL_SUCCESS != (ret = opal_pmix.store_local(ORTE_PROC_MY_DAEMON, &val))) {
+            ORTE_ERROR_LOG(ret);
+            val.key = NULL;
+            val.data.string = NULL;
+            OBJ_DESTRUCT(&val);
+            error = "store DAEMON URI";
+            goto error;
+        }
+        val.key = NULL;
+        val.data.string = NULL;
+        OBJ_DESTRUCT(&val);
+    }
+
+    /* setup the errmgr */
+    if (ORTE_SUCCESS != (ret = orte_errmgr_base_select())) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_errmgr_base_select";
+        goto error;
+    }
+
+    /* setup process binding */
+    if (ORTE_SUCCESS != (ret = orte_ess_base_proc_binding())) {
+        error = "proc_binding";
+        goto error;
+    }
+
+    /* this needs to be set to enable debugger use when direct launched */
+    if (NULL == orte_process_info.my_daemon_uri) {
+        orte_standalone_operation = true;
+    }
+
+    /* set max procs */
+    if (orte_process_info.max_procs < orte_process_info.num_procs) {
+        orte_process_info.max_procs = orte_process_info.num_procs;
+    }
+
+    /* push our hostname so others can find us, if they need to - the
+     * native PMIx component will ignore this request as the hostname
+     * is provided by the system */
+    OPAL_MODEX_SEND_VALUE(ret, OPAL_PMIX_GLOBAL, OPAL_PMIX_HOSTNAME, orte_process_info.nodename, OPAL_STRING);
+    if (ORTE_SUCCESS != ret) {
+        error = "db store hostname";
+        goto error;
+    }
+
+    /* if we are an ORTE app - and not an MPI app - then
+     * we need to exchange our connection info here.
+     * MPI_Init has its own modex, so we don't need to do
+     * two of them. However, if we don't do a modex at all,
+     * then processes have no way to communicate
+     *
+     * NOTE: only do this when the process originally launches.
+     * Cannot do this on a restart as the rest of the processes
+     * in the job won't be executing this step, so we would hang
+     */
+    if (ORTE_PROC_IS_NON_MPI && !orte_do_not_barrier) {
+        /* need to commit the data before we fence */
+        opal_pmix.commit();
+        if (ORTE_SUCCESS != (ret = opal_pmix.fence(NULL, 0))) {
+            error = "opal_pmix.fence() failed";
+            goto error;
+        }
     }
 
     return ORTE_SUCCESS;
 
- error:
+  error:
     if (ORTE_ERR_SILENT != ret && !orte_report_silent_errors) {
         orte_show_help("help-orte-runtime.txt",
                        "orte_init:startup:internal-failure",
@@ -291,8 +472,6 @@ static int rte_init(void)
 
 static int rte_finalize(void)
 {
-    int ret;
-
     /* remove the envars that we pushed into environ
      * so we leave that structure intact
      */
@@ -311,10 +490,9 @@ static int rte_finalize(void)
         unsetenv("PMIX_SERVER_URI");
         unsetenv("PMIX_SECURITY_MODE");
     }
-    /* use the default procedure to finish */
-    if (ORTE_SUCCESS != (ret = orte_ess_base_app_finalize())) {
-        ORTE_ERROR_LOG(ret);
-    }
+    /* close frameworks */
+    (void) mca_base_framework_close(&orte_filem_base_framework);
+    (void) mca_base_framework_close(&orte_errmgr_base_framework);
 
     /* mark us as finalized */
     if (NULL != opal_pmix.finalize) {
@@ -322,12 +500,18 @@ static int rte_finalize(void)
         (void) mca_base_framework_close(&opal_pmix_base_framework);
     }
 
+    (void) mca_base_framework_close(&orte_state_base_framework);
+    orte_session_dir_finalize(ORTE_PROC_MY_NAME);
+
+    /* cleanup the process info */
+    orte_proc_info_finalize();
+
     /* release the event base */
     if (progress_thread_running) {
         opal_progress_thread_finalize(NULL);
         progress_thread_running = false;
     }
-    return ret;
+    return ORTE_SUCCESS;
 }
 
 #define ORTE_URI_MSG_LGTH   256
@@ -584,3 +768,25 @@ static int fork_hnp(void)
         return ORTE_SUCCESS;
     }
 }
+
+static void rte_abort(int status, bool report)
+{
+    struct timespec tp = {0, 100000};
+
+    OPAL_OUTPUT_VERBOSE((1, orte_ess_base_framework.framework_output,
+                         "%s ess:singleton:abort: abort with status %d",
+                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                         status));
+
+    /* PMI doesn't like NULL messages, but our interface
+     * doesn't provide one - so rig one up here
+     */
+    opal_pmix.abort(status, "N/A", NULL);
+
+    /* provide a little delay for the PMIx thread to
+     * get the info out */
+    nanosleep(&tp, NULL);
+
+    /* Now Exit */
+    _exit(status);
+}
