@@ -16,7 +16,7 @@
  * Copyright (c) 2010      Oracle and/or its affiliates.  All rights
  *                         reserved.
  * Copyright (c) 2009-2016 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2011      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2011-2020 IBM Corporation.  All rights reserved.
  * Copyright (c) 2015-2017 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
@@ -222,6 +222,14 @@ static int rsh_component_register(void)
                                             MCA_BASE_VAR_SCOPE_READONLY,
                                             &mca_plm_rsh_component.pass_libpath);
 
+    mca_plm_rsh_component.chdir = NULL;
+    (void) mca_base_component_var_register (c, "chdir",
+                                            "Change working directory after rsh/ssh, but before exec of orted",
+                                            MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
+                                            OPAL_INFO_LVL_2,
+                                            MCA_BASE_VAR_SCOPE_READONLY,
+                                            &mca_plm_rsh_component.chdir);
+
     return ORTE_SUCCESS;
 }
 
@@ -362,6 +370,10 @@ char **orte_plm_rsh_search(const char* agent_list, const char *path)
     char **tokens, *tmp;
     char cwd[OPAL_PATH_MAX];
 
+    if (NULL == agent_list && NULL == mca_plm_rsh_component.agent) {
+        return NULL;
+    }
+
     if (NULL == path) {
         getcwd(cwd, OPAL_PATH_MAX);
     } else {
@@ -413,6 +425,14 @@ static int rsh_launch_agent_lookup(const char *agent_list, char *path)
     char *bname;
     int i;
 
+    if (NULL == agent_list && NULL == mca_plm_rsh_component.agent) {
+        OPAL_OUTPUT_VERBOSE((5, orte_plm_base_framework.framework_output,
+                             "%s plm:rsh_lookup on agent (null) path %s - No agent specified.",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                             (NULL == path) ? "NULL" : path));
+        return ORTE_ERR_NOT_FOUND;
+    }
+
     OPAL_OUTPUT_VERBOSE((5, orte_plm_base_framework.framework_output,
                          "%s plm:rsh_lookup on agent %s path %s",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
