@@ -13,6 +13,8 @@
  * Copyright (c) 2007-2013 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2015-2017 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2017      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -172,8 +174,32 @@ void orte_rml_base_process_msg(int fd, short flags, void *cbdata)
 
     /* if this message is just to warmup the connection, then drop it */
     if (ORTE_RML_TAG_WARMUP_CONNECTION == msg->tag) {
-        OBJ_RELEASE(msg);
-        return;
+        if (!orte_nidmap_communicated) {
+            opal_buffer_t * buffer = OBJ_NEW(opal_buffer_t);
+            int rc;
+            if (NULL == buffer) {
+                ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
+                return;
+            }
+            assert (NULL != orte_node_regex);
+
+            if (ORTE_SUCCESS != (rc = opal_dss.pack(buffer, &orte_node_regex, 1, OPAL_STRING))) {
+                ORTE_ERROR_LOG(rc);
+                OBJ_RELEASE(buffer);
+                return;
+            }
+
+            if (ORTE_SUCCESS != (rc = orte_rml.send_buffer_nb(orte_mgmt_conduit,
+                                                              &msg->sender, buffer,
+                                                              ORTE_RML_TAG_NODE_REGEX_REPORT,
+                                                              orte_rml_send_callback, NULL))) {
+                ORTE_ERROR_LOG(rc);
+                OBJ_RELEASE(buffer);
+                return;
+            }
+            OBJ_RELEASE(msg);
+            return;
+        }
     }
 
     /* see if we have a waiting recv for this message */
