@@ -20,6 +20,7 @@
 #include "oshmem/include/pshmem.h"
 #pragma weak shmem_n_pes = pshmem_n_pes
 #pragma weak shmem_my_pe = pshmem_my_pe
+#pragma weak shmem_query_thread = pshmem_query_thread
 #pragma weak _num_pes = p_num_pes
 #pragma weak _my_pe = p_my_pe
 #include "oshmem/shmem/c/profile/defines.h"
@@ -60,3 +61,8 @@ int my_pe(void)
     RUNTIME_CHECK_INIT();
     return oshmem_my_proc_id();
 }
+void shmem_query_thread(int *provided)
+{
+    RUNTIME_CHECK_INIT();
+    (*provided) = oshmem_mpi_thread_provided;
+}
