@@ -27,6 +27,7 @@
 #define N 331
 
 uint32_t remote_arch = 0xffffffff;
+bool report_all_errors = true;
 
 struct foo_t {
     int i[3];
@@ -38,26 +39,28 @@ struct pfoo_t {
     double d[2];
 } pfoo = {0}, *pbar = NULL;
 
-static void print_hex(void* ptr, int count, int space)
+static void print_hex(void* ptr, int count, char* epilog, char* prolog)
 {
-    for( int i = 0; i < count; i++ ) {
+    if ( NULL != epilog) fprintf(stderr, "%s", epilog);
+    for ( int i = 0; i < count; i++ ) {
         fprintf(stderr, "%02x", (unsigned int)(((unsigned char*)ptr)[i]));
     }
-    if(space) fprintf(stderr, " ");
+    if (NULL != prolog) fprintf(stderr, "%s", prolog);
 }
 
-static void print_bar_pbar(struct foo_t* bar, struct pfoo_t* pbar)
+static void print_bar_pbar(struct foo_t* _bar, struct pfoo_t* _pbar)
 {
-    print_hex(&bar->i[0], sizeof(int), 1);
-    print_hex(&bar->i[2], sizeof(int), 1);
-    print_hex(&bar->d[0], sizeof(double), 1);
-    print_hex(&bar->d[2], sizeof(double), 1);
-    fprintf(stderr, "\n");
-    print_hex(&pbar->i[0], sizeof(int), 1);
-    print_hex(&pbar->i[1], sizeof(int), 1);
-    print_hex(&pbar->d[0], sizeof(double), 1);
-    print_hex(&pbar->d[1], sizeof(double), 1);
-    fprintf(stderr, "\n");
+    print_hex(&_bar->i[0], sizeof(int), NULL, " ");
+    print_hex(&_bar->i[1], sizeof(int), "[", "] ");
+    print_hex(&_bar->i[2], sizeof(int), NULL, " ");
+    print_hex(&_bar->d[0], sizeof(double), NULL, " ");
+    print_hex(&_bar->d[1], sizeof(double), "[", "] ");
+    print_hex(&_bar->d[2], sizeof(double), NULL, "\n");
+
+    print_hex(&_pbar->i[0], sizeof(int), NULL, "            ");
+    print_hex(&_pbar->i[1], sizeof(int), NULL, " ");
+    print_hex(&_pbar->d[0], sizeof(double), NULL, "                    ");
+    print_hex(&_pbar->d[1], sizeof(double), NULL, "\n");
 }
 
 static void print_stack(opal_convertor_t* conv)
@@ -72,7 +75,7 @@ static void print_stack(opal_convertor_t* conv)
     printf("\n");
 }
 
-static int testcase(ompi_datatype_t * newtype, size_t arr[10][2]) {
+static int testcase(ompi_datatype_t * newtype, size_t arr[][2]) {
     int i, j, errors = 0;
     struct iovec a;
     unsigned int iov_count;
@@ -99,7 +102,7 @@ static int testcase(ompi_datatype_t * newtype, size_t arr[10][2]) {
         return OMPI_ERROR;
     }
 
-    for (i=0; arr[i][0] != 0; i++) {
+    for ( i = 0; 0 != arr[i][0]; i++) {
         /* add some garbage before and after the source data */
         a.iov_base = malloc(arr[i][0]+2048);
         if (NULL == a.iov_base) {
@@ -129,11 +132,36 @@ static int testcase(ompi_datatype_t * newtype, size_t arr[10][2]) {
             bar[j].d[0] != pbar[j].d[0] ||
             bar[j].d[1] != 0.0 ||
             bar[j].d[2] != pbar[j].d[1]) {
-            if(0 == errors) {
+            if(0 == errors || report_all_errors) {
+                ptrdiff_t displ;
+                char* error_location = "in gaps";
+                if (bar[j].i[0] != pbar[j].i[0]) {
+                    displ = (char*)&bar[j].i[0] - (char*)&bar[0];
+                    error_location = "i[0]";
+                } else if (bar[j].i[2] != pbar[j].i[1]) {
+                    displ = (char*)&bar[j].i[1] - (char*)&bar[0];
+                    error_location = "i[2]";
+                } else if (bar[j].d[0] != pbar[j].d[0]) {
+                    displ = (char*)&bar[j].d[0] - (char*)&bar[0];
+                    error_location = "d[0]";
+                } else if (bar[j].d[2] != pbar[j].d[1]) {
+                    displ = (char*)&bar[j].d[1] - (char*)&bar[0];
+                    error_location = "d[2]";
+                } else {
+                    displ = (char*)&bar[j] - (char*)&bar[0];
+                }
+                for (i = 0; 0 != arr[i][0]; i++) {
+                    if( (displ >= arr[i][1]) && (displ <= (arr[i][1] + arr[i][0])) ) {
+                        fprintf(stderr, "Problem encountered %li bytes into the %d unpack [%"PRIsize_t":%"PRIsize_t"]\n",
+                                displ - arr[i][1], i, arr[i][1], arr[i][0]);
+                        break;
+                    }
+                }
+
                 (void)opal_datatype_dump(&newtype->super);
-                fprintf(stderr, "ERROR ! position=%d/%d, ptr = %p"
+                fprintf(stderr, "ERROR ! struct %d/%d in field %s, ptr = %p"
                         " got (%d,%d,%d,%g,%g,%g) expected (%d,%d,%d,%g,%g,%g)\n",
-                        j, N, (void*)&bar[j],
+                        j, N, error_location, (void*)&bar[j],
                         bar[j].i[0],
                         bar[j].i[1],
                         bar[j].i[2],
@@ -147,6 +175,7 @@ static int testcase(ompi_datatype_t * newtype, size_t arr[10][2]) {
                         0.0,
                         pbar[j].d[1]);
                 print_bar_pbar(&bar[j], &pbar[j]);
+                if( report_all_errors ) fprintf(stderr, "\n\n");
             }
             errors++;
         }
@@ -198,13 +227,13 @@ static int unpack_ooo(void)
  */
     size_t test1[9][2] = {
         {992, 0},
-        {1325, 992},
-        {992, 2317},
-        {992, 3309},
-        {992, 4301},
-        {992, 5293},
-        {992, 6285},
-        {667, 7277},
+        {1325, 0 + 992},
+        {992, 992 + 1325 /* = 2317 */},
+        {992, 2317 + 992 /* = 3309 */},
+        {992, 3309 + 992 /* = 4301 */},
+        {992, 4301 + 992 /* = 5293 */},
+        {992, 5293 + 992 /* = 6285 */},
+        {667, 6285 + 992 /* = 7277 */},
         {0, -1},
     };
 
